from __future__ import annotations

from tcsoa.gen.Cae._2009_10.SimulationProcessManagement import InputObjectsStructure
from typing import List
from tcsoa.base import TcService


class SimulationProcessManagementService(TcService):

    @classmethod
    def launchSimulationTool(cls, inputObjects: List[InputObjectsStructure], toolName: str, launchType: str, itemCreationOption: str, datasetCreationOption: str, plmxmlExportFileName: str, workingDirectory: str, runtimeArguments: List[str]) -> bool:
        """
        This operation will launch the simulation tool as per the pre-defined configuration. The configuration contains
        details about the tool launch type, export and import rules for the files, notification, and cleanup. The
        inputs to the operation will be an array of InputObjectStructure, each containing one or more ItemRevision
        objects, name of the tool to be launched and the launch type. The launch type could be "LOCAL" or "REMOTE".
        Based on the input values of parameters itemCreationOption and datasetCreationOption, ItemRevision and Dataset
        objects will be created or modified.
        
        It is pre-requisite that the Simulation Administrator or user with DBA privileges must configure the simulation
        tool and store it in XML format as a named reference with the Dataset name specified in the preference
        CAE_simulation_tool_config_dsname.
        
        To use this operation, the user should have either a simulation_author or rtt_author license.
        
        
        Use cases:
        Use Case 1: Launch Simulation Tool with "As_Needed" Item Creation Option
        When this operation is executed with Item creation setting "As_Needed", no new Teamcenter Item will be created
        if there exists one as per the process output configuration. If found none, new Item will be created as per the
        process output configuration.
        
        Use Case 2: Launch Simulation Tool with "Always" Item Creation Option
        When this operation is executed with Item creation setting "Always", new Teamcenter Item objects will be
        created as per the process output configuration.
        
        Use Case 3: Launch Simulation Tool with "Never" Item Creation Option
        When this operation is executed with Item creation setting "Never" and if no existing Item is found as per the
        process output configuration, no new Teamcenter Item objects will be created.
        
        Use Case 4: Launch Simulation Tool with "As_Needed" Dataset Creation Option
        When this operation is executed with Dataset creation setting "As Needed", no new Teamcenter Dataset will be
        created if there exists one as per the process output configuration. If found none, new Dataset will be created
        as per the process output configuration.
        
        Use Case 5: Launch Simulation Tool with "Always" Dataset Creation Option
        When this operation is executed with Dataset creation setting "Always", new Teamcenter Dataset objects will be
        created as per the process output configuration.
        
        Use Case 6: Launch Simulation Tool with "Never" Dataset Creation Option
        When this operation is executed with Dataset creation setting "Never" and if no existing Dataset is found as
        per the process output configuration, no new Teamcenter Dataset objects will be created.
        """
        return cls.execute_soa_method(
            method_name='launchSimulationTool',
            library='Cae',
            service_date='2011_06',
            service_name='SimulationProcessManagement',
            params={'inputObjects': inputObjects, 'toolName': toolName, 'launchType': launchType, 'itemCreationOption': itemCreationOption, 'datasetCreationOption': datasetCreationOption, 'plmxmlExportFileName': plmxmlExportFileName, 'workingDirectory': workingDirectory, 'runtimeArguments': runtimeArguments},
            response_cls=bool,
        )



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class TransformDataResponse(TcBaseObj):
    """
    The TransformDataResponse holds the FMS tickets of the output file, the log file and also contains the partial
    errors that are used to report any partial failures.
    
    :var outputFileTicket: The FMS ticket is used to get the transformed output file.
    :var logFileTicket: The FMS ticket is sued to get the generated transform log file. All the warning and error
    messages in the transform process are recorded in the log file.
    :var serviceData: Service data contains user facing partial errors which are returned to the client. For the
    complete list of errors and warnings during the transform process, users need to check the log file.
    """
    outputFileTicket: str = ''
    logFileTicket: str = ''
    serviceData: ServiceData = None



from tcsoa.gen.Internal_DebugMonitor._2011_06.services import DebugMonitorService as imp0
from tcsoa.gen.Internal_DebugMonitor._2011_06.services import PerformanceMonitorService as imp1
from tcsoa.gen.Internal_DebugMonitor._2011_06.services import DebugLoggingService as imp2
from tcsoa.gen.Internal_DebugMonitor._2015_07.services import DebugLoggingService as imp3
from tcsoa.gen.Internal_DebugMonitor._2014_06.services import JournalBasedTestingService as imp4
from tcsoa.gen.Internal_DebugMonitor._2019_06.services import DebugLoggingService as imp5
from tcsoa.gen.Internal_DebugMonitor._2015_10.services import JournalBasedTestingService as imp6
from tcsoa.base import TcService


class DebugMonitorService(TcService):
    clearAllHistory = imp0.clearAllHistory
    clearHistory = imp0.clearHistory
    getAlertInformation = imp0.getAlertInformation
    getMetricMetaData = imp0.getMetricMetaData
    getOperationalHealthData = imp0.getOperationalHealthData
    getTcServerMonitoringMode = imp0.getTcServerMonitoringMode
    getValidMetrics = imp0.getValidMetrics
    reloadConfiguration = imp0.reloadConfiguration
    setMetricConfig = imp0.setMetricConfig
    setTcServerMonitoringMode = imp0.setTcServerMonitoringMode


class PerformanceMonitorService(TcService):
    clearPerformanceInformation = imp1.clearPerformanceInformation
    getPerformanceInformation = imp1.getPerformanceInformation
    monitorPerformance = imp1.monitorPerformance


class DebugLoggingService(TcService):
    dumpStatistics = imp2.dumpStatistics
    getCheckingLevel = imp2.getCheckingLevel
    getDebugStatus = imp2.getDebugStatus
    getJournalFile = imp2.getJournalFile
    getJournalingStatus = imp2.getJournalingStatus
    getLoggerLevels = imp2.getLoggerLevels
    getModuleJournalStatus = imp2.getModuleJournalStatus
    getPerformanceJournalLevel = imp3.getPerformanceJournalLevel
    getSQLLogging = imp2.getSQLLogging
    getStatisticsStatus = imp2.getStatisticsStatus
    getSyslogFile = imp2.getSyslogFile
    getTAOLogLevel = imp2.getTAOLogLevel
    resetStatistics = imp2.resetStatistics
    setCheckingLevel = imp2.setCheckingLevel
    setJournalEntries = imp2.setJournalEntries
    setLoggerLevels = imp2.setLoggerLevels
    setModuleJournalStatus = imp2.setModuleJournalStatus
    setPerformanceJournalLevel = imp3.setPerformanceJournalLevel
    setSQLLogging = imp2.setSQLLogging
    setStatistics = imp2.setStatistics
    setTAOLogLevel = imp2.setTAOLogLevel
    startLogging = imp5.startLogging
    stopLogging = imp5.stopLogging


class JournalBasedTestingService(TcService):
    initializeJBT = imp4.initializeJBT
    terminateJBT = imp6.terminateJBT
    validatePropertyValues = imp4.validatePropertyValues



from tcsoa.gen.Ai._2006_03.services import AiService as imp0
from tcsoa.gen.Ai._2008_06.services import AiService as imp1
from tcsoa.gen.Ai._2014_12.services import AiService as imp2
from tcsoa.gen.Ai._2013_05.services import AiService as imp3
from tcsoa.gen.Ai._2012_09.services import AiService as imp4
from tcsoa.gen.Ai._2013_12.services import AiService as imp5
from tcsoa.gen.Ai._2010_09.services import AiService as imp6
from tcsoa.gen.Ai._2008_05.services import AiService as imp7
from tcsoa.gen.Ai._2007_12.services import AiService as imp8
from tcsoa.gen.Ai._2009_06.services import AiService as imp9
from tcsoa.gen.Ai._2018_06.services import AiService as imp10
from tcsoa.gen.Ai._2009_10.services import AiService as imp11
from tcsoa.base import TcService


class AiService(TcService):
    commitFiles = imp0.commitFiles
    commitStructureFile = imp0.commitStructureFile
    compareConfigurationContexts = imp1.compareConfigurationContexts
    createApplicationInterfaceRecords = imp2.createApplicationInterfaceRecords
    createProjects = imp0.createProjects
    createPublishRequest = imp0.createPublishRequest
    deleteProjects = imp0.deleteProjects
    deleteRequests = imp0.deleteRequests
    endExchange = imp0.endExchange
    findRequestOnAiWithReferences = imp3.findRequestOnAiWithReferences
    findRequests = imp4.findRequests
    findRequestsWithDependencies = imp5.findRequestsWithDependencies
    generateAndEvaluateStructure = imp6.generateAndEvaluateStructure
    generateArrangements = imp7.generateArrangements
    generateFullSyncRequest = imp0.generateFullSyncRequest
    generateScopedMultipleStructure = imp8.generateScopedMultipleStructure
    generateScopedMultipleStructure2 = imp1.generateScopedMultipleStructure2
    generateScopedMultipleStructure3 = imp9.generateScopedMultipleStructure3
    generateScopedSyncRequest = imp8.generateScopedSyncRequest
    generateScopedSyncRequest2 = imp1.generateScopedSyncRequest2
    generateStructure = imp0.generateStructure
    getFileReadTickets = imp0.getFileReadTickets
    getFileWriteTickets = imp0.getFileWriteTickets
    getMappedApplicationRefs = imp2.getMappedApplicationRefs
    getNextApprovedRequest = imp0.getNextApprovedRequest
    getObjectsByApplicationRefs = imp10.getObjectsByApplicationRefs
    getPersistentObjects = imp9.getPersistentObjects
    getProjects = imp0.getProjects
    getProjects2 = imp4.getProjects
    getProjectsInfo = imp0.getProjectsInfo
    getProjectsInfo2 = imp4.getProjectsInfo2
    getPropertiesOfObjects = imp0.getPropertiesOfObjects
    getPropertyValues = imp11.getPropertyValues
    getRequestsForProject = imp0.getRequestsForProject
    getRequestsInfo = imp0.getRequestsInfo
    getRequestsInfo2 = imp4.getRequestsInfo2
    getStructureReadTicket = imp0.getStructureReadTicket
    getStructureWriteTicket = imp0.getStructureWriteTicket
    objectsExist = imp0.objectsExist
    processPublishRequest = imp0.processPublishRequest
    processStructure = imp0.processStructure
    setExchangeMessage = imp0.setExchangeMessage
    setProjectsInfo = imp0.setProjectsInfo
    setProjectsInfo2 = imp4.setProjectsInfo
    setRequestsInfo = imp4.setRequestsInfo
    startExchange = imp0.startExchange



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetViewableDataResponse(TcBaseObj):
    """
    Output structure for getViewableData operation.
    
    :var output: List of ViewerData objects, one for each input object.
    :var serviceData: The service data contains any partial errors which may have been encountered during processing.
    All BusinessObjects which are returned are added to the serviceData plainObjects list.
    """
    output: List[ViewerData] = ()
    serviceData: ServiceData = None


@dataclass
class TypeToNamedRefPair(TcBaseObj):
    """
    The 'TypeToNamedRefPair' structure contains the Named Reference type string, as well as a list of file tickets that
    Named Reference type, so the client may download the files.
    
    :var namedRefType: The named reference type string. For example for an MSWord dataset, it might have a Named
    Reference with type word.
    :var fileTickets: List of file ticket strings.
    """
    namedRefType: str = ''
    fileTickets: List[str] = ()


@dataclass
class ViewerData(TcBaseObj):
    """
    The ViewerData structure contains all the data required in order for a client to present viewable data for the
    input object in a client.
    
    :var inputObj: The input object which this output represents.
    :var viewerID: The target viewer ID to load in the client.
    :var viewableObj: The target object to view.  If it's a dataset, then the viewableNameRefs field may be populated
    depending on the defaultViewerConfig.VIEWERCONFIG (or <viewer config id>.VIEWERCONFIG) preference that defines the
    rules for viewing an object.
    :var traversal: The list of objects traversed from the input object to the viewable object.
    :var viewableNamedRefs: List of viewable named references.  This structure includes the named reference string and
    a list of 'fileTickets' so the client may download the files.
    
    This field is only populated if the 'viewableObj' is a Dataset type, and the rules define its named references as
    being viewable.
    """
    inputObj: BusinessObject = None
    viewerID: str = ''
    viewableObj: BusinessObject = None
    traversal: List[BusinessObject] = ()
    viewableNamedRefs: List[TypeToNamedRefPair] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, CadAttrMappingDefinition
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from typing import Dict


@dataclass
class MappedDatasetAttrPropertyInfoForNX(TcBaseObj):
    """
    Contains the resolved attribute information.
    
    :var cadAttrMappingDefinition: 'CadAttributeMappingDefinition' object reference associated with the resolved
    property object.
    :var resolvedObject: Object reference of object holding mapped attribute value.
    :var resolvedPropertyName: The property name of the mapped object holding the attribute value of interest resulting
    from evaluation of a dataset CAD attribute mapping definition.
    """
    cadAttrMappingDefinition: CadAttrMappingDefinition = None
    resolvedObject: BusinessObject = None
    resolvedPropertyName: str = ''


@dataclass
class NxFrozenAndModifiableInfo(TcBaseObj):
    """
    Contains additional NX-specific information about the "frozen" and "modifiable" values of the resolved attributes.
    
    :var isModifiable: Whether the mapped Teamcenter property is modifiable in the NX specific semantics.
    :var isFrozen: Whether the mapped Teamcenter property is frozen in the NX specific semantics.
    """
    isModifiable: bool = False
    isFrozen: bool = False


@dataclass
class NxResolvedConstOrPrefAttrInfo(TcBaseObj):
    """
    Contains additional NX specific information for attributes whose mapping definitions are of ATTRMAP_preference or
    ATTRMAP_constant type.
    
    
    :var isFrozen: Whether the mapped Teamcenter property is frozen in the NX specific semantics.
    :var value: mapped attr value.
    """
    isFrozen: bool = False
    value: str = ''


@dataclass
class ResolveAttrMappingsForNXResponse(TcBaseObj):
    """
    Contains the response for 'resolveAttrMappingsForNX'
    
    :var resolvedMappingsMap: This is a map containing the successfully mapped property information. The keys are the
    input 'clientIds' and the values are the output 'MappedNxDatasetAttrPropertyInfo' structures.
    :var frozenAndModifiableInfoMap: This is a map containing the NX specific frozen and modifiable value of
    successfully resolved attributes. The keys are the input 'clientIds' and the values are the output
    'NxFrozenAndModifiableInfo' structures.
    :var resolvedConstOrPrefAttrInfoMap: This is a map containing the NX specific frozen value and the resolved value
    for attributes whose mappinge definitions are of ATTRMAP_preference or ATTRMAP_constant type.
    The keys are the input 'clientIds' and the values are the output 'NxResolvedConstOrPrefAttrInfo' structures.
    :var serviceData: Service data contains all objects returned by the service. Also contains error messages. . If no
    entry is found in the returned maps then an error may have occurred. The client should look for it mapped to a
    Partial Error. Look for a Partial error with with an id of NX: + clientId for error occurs in NX specific
    processing. If none of the above exists, locate a Partial Error, this indicates that the processing of the dataset
    or the item revision (where the attribute belongs) failed. Iterate through the input indexes of the list of
    ResolveAttrMappingsInfo and look for index mapped to the Partial Errors of the Service Data.
    Error reported by this service :
    215119 No Dataset or Item Revision specified
    215074 Invalid Dataset
    215075 Invalid Item Revision
    215076 Invalid CadAttrMappingDefinitionInfo object
    215200 No clientId specified
    215201 Duplicate clientId specified
    """
    resolvedMappingsMap: ResolveAttrMappingsOutputMapForNX = None
    frozenAndModifiableInfoMap: NxFrozenAndModifiableInfoMap = None
    resolvedConstOrPrefAttrInfoMap: NxResolvedConstOrPrefAttrInfoMap = None
    serviceData: ServiceData = None


"""
Map of string client ids to 'NxFrozenAndModifiableInfo' values (string, 'NxFrozenAndModifiableInfo').
"""
NxFrozenAndModifiableInfoMap = Dict[str, NxFrozenAndModifiableInfo]


"""
Map of string client ids to 'NxResolvedConstOrPrefAttrInfo' values (string, 'NxResolvedConstOrPrefAttrInfo').
"""
NxResolvedConstOrPrefAttrInfoMap = Dict[str, NxResolvedConstOrPrefAttrInfo]


"""
Map of string client ids to 'MappedDatasetAttrPropertyInfoForNX' values (string, 'MappedDatasetAttrPropertyInfoForNX').
"""
ResolveAttrMappingsOutputMapForNX = Dict[str, MappedDatasetAttrPropertyInfoForNX]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Item, WorkspaceObject, ItemRevision
from typing import Dict, List
from tcsoa.gen.Internal.Visualization._2008_06.DataManagement import NamedRefsInDataset
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GatherBasedDatasetInfo(TcBaseObj):
    """
    Base dataset information.
    
    :var uid: the base Dataset object uid.   The base Dataset is the Dataset where the markup Datasets are related to.
    :var name: the base Dataset object name.
    :var additionalInfo: the additional information will have the followings: 
    RequestMode=VIEW or MARKUP
    PrintMode=PRINT or NOPRINT
    AccessMode=READ or WRITE
    Arguments=(empty)
    FilePath=(base file name, for example, MyFile.pdf)
    DataSegmentFileType=oai (if the file is a Microsoft office file) or file extension for other file type.
    DataSegment=(this contains all the above information, but all the information are in a single string. 
    Additionally, it also contains the server information: host path, SSO application id, and SSO loginURL)
    :var namedRefInfos: the list of base dataset named references.
    :var controlObj: the control object structure with uid, name, and any additional information.
    :var markupList: the list of the related markup structures that are related to the base Dataset.
    """
    uid: str = ''
    name: str = ''
    additionalInfo: GatherKeyValueMap = None
    namedRefInfos: List[NamedRefsInDataset] = ()
    controlObj: GatherMarkupControlObject = None
    markupList: List[GatherMarkupInfo] = ()


@dataclass
class GatherBasedDatasetTypeInfo(TcBaseObj):
    """
    Base dataset type information.
    
    :var relatedDatasets: the list of base Datasets of the same type.
    :var toolPreferences: the list of markup tool preferences.
    :var name: the Dataset type name.
    """
    relatedDatasets: List[GatherBasedDatasetInfo] = ()
    toolPreferences: List[str] = ()
    name: str = ''


@dataclass
class GatherBusinessDataInfo(TcBaseObj):
    """
    Item and ItemRevision information or Dataset type information or both.
    
    :var gatherItemInfo: the structure of Item and Item Revision information.
    :var gatherDatasetTypeInfos: the list of structures of the DatasetType and Datasets information.
    """
    gatherItemInfo: GatherItemInfo = None
    gatherDatasetTypeInfos: List[GatherBasedDatasetTypeInfo] = ()


@dataclass
class GatherInputInfo(TcBaseObj):
    """
    the input information.
    
    :var id: the id can be a WorkspaceObject, an Item, an Item Revision, a Dataset or a markup control object (a markup
    control object is any WorkspaceObject that has the business object Fnd0MarkupControlObject constant is defined as
    true, default is false).  The input id is required.  If empty, invalid object error is returned.
    :var item: the related Item to the input id above.  This can be empty.
    :var itemRev: the related Item Revision to the input id above.  This can be empty.
    :var controlObj: the related markup control object to the input id above.  This can be empty.
    :var idAdditionalInfo: the map of additional information in the form of key value pair strings.  This can be empty.
      Currently, the system only processes the input with key=MARKUPTOOL and value=(tool name).
    """
    id: WorkspaceObject = None
    item: Item = None
    itemRev: ItemRevision = None
    controlObj: WorkspaceObject = None
    idAdditionalInfo: GatherKeyValueMap = None


@dataclass
class GatherItemInfo(TcBaseObj):
    """
    Item and ItemRevision information.
    
    :var itemUid: the Item object uid.
    :var itemName: the Item object name.
    :var itemRevUid: the Item Revision object uid.
    :var itemRevName: the Item Revision object name.
    :var toolPreferences: the list of markup tool preferences.
    """
    itemUid: str = ''
    itemName: str = ''
    itemRevUid: str = ''
    itemRevName: str = ''
    toolPreferences: List[str] = ()


@dataclass
class GatherMarkupControlObject(TcBaseObj):
    """
    Markup control object information.
    
    :var uid: the control object uid.  A markup control object is any WorkspaceObject that has the Business Object
    Fnd0MarkupControlObject constant is defined as true, default is false.
    :var name: the control object name.
    :var additionalInfo: any additional information in the form of key value pair strings.  This is for future use.
    """
    uid: str = ''
    name: str = ''
    additionalInfo: GatherKeyValueMap = None


@dataclass
class GatherMarkupInfo(TcBaseObj):
    """
    Markup dataset information.
    
    :var uid: the markup Dataset object uid.
    :var name: the markup Dataset object name.
    :var additionalInfo: the additional information will have the followings: 
    MarkupFormat=(file extension, example: fdf)
    AccessMode=READ or WRITE
    CreatedOn=(yyyy mm dd.hh.mm.ss)
    :var namedRefInfos: the list of markup Dataset named references.
    """
    uid: str = ''
    name: str = ''
    additionalInfo: GatherKeyValueMap = None
    namedRefInfos: List[NamedRefsInDataset] = ()


@dataclass
class GatherResponse(TcBaseObj):
    """
    Gather markup return information.
    
    :var businessDataList: the list of the structures of the Item, DatasetType information.
    :var svcData: the service data that will have the failed objects and errors.
    """
    businessDataList: List[GatherBusinessDataInfo] = ()
    svcData: ServiceData = None


@dataclass
class GatherServerInfo(TcBaseObj):
    """
    Server information.
    
    :var protocol: the connection protocal to the server.  It can be http or iiop.
    :var hostPath: the host path.
    :var serverMode: the server mode character.  2 is for two tier.   4 is for four tier.
    :var useSso: if the client uses Single Sign On (SSO) environment to connect to the server, this is set to true,
    otherwise false.
    :var tccsEnvironment: the SSO environment name.
    :var ssoInfo: the structure of the SSO service url and id.
    :var additionalInfo: the map of additional information in the form of key value pair strings.  This is currently
    not used (future).
    """
    protocol: str = ''
    hostPath: str = ''
    serverMode: str = ''
    useSso: bool = False
    tccsEnvironment: str = ''
    ssoInfo: GatherSsoInfo = None
    additionalInfo: GatherKeyValueMap = None


@dataclass
class GatherSessionInfo(TcBaseObj):
    """
    Session information.
    
    :var sessionDescriminator: the Client Server session discriminator to connect to the existing specified session.
    :var sessionAdditionalInfo: the map of additional information in the form of key value pair strings.
    """
    sessionDescriminator: str = ''
    sessionAdditionalInfo: GatherKeyValueMap = None


@dataclass
class GatherSsoInfo(TcBaseObj):
    """
    Single Sign On (SSO) information.
    
    :var loginServiceUrl: the login service url.
    :var appId: the application id of the Teamcenter server in the SSO environment.
    """
    loginServiceUrl: str = ''
    appId: str = ''


@dataclass
class GatherUserAgentDataInfo(TcBaseObj):
    """
    User application information.
    
    :var userApplication: the client name that initiates the launch.
    :var userAppVersion: the version of the client.
    :var userAdditionalInfo: the map of additional information in the form of key value pair strings.  This is
    currently not used (future).
    """
    userApplication: str = ''
    userAppVersion: str = ''
    userAdditionalInfo: GatherKeyValueMap = None


"""
Key value map of string, string.
"""
GatherKeyValueMap = Dict[str, str]



from tcsoa.gen.Internal_Ai._2008_06.services import AiService as imp0
from tcsoa.gen.Internal_Ai._2016_03.services import AiService as imp1
from tcsoa.gen.Internal_Ai._2016_04.services import AiService as imp2
from tcsoa.base import TcService


class AiService(TcService):
    beginGenerateMonolithicJt = imp0.beginGenerateMonolithicJt
    endGenerateMonolithicJt = imp0.endGenerateMonolithicJt
    getApplicationInterfaceTypes = imp1.getApplicationInterfaceTypes
    invoke = imp0.invoke
    invoke2 = imp2.invoke2



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.Manufacturing._2008_12.Core import CheckoutInput, CheckinInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class CoreService(TcService):

    @classmethod
    def checkinForProcessSimulate(cls, rootLines: List[BusinessObject], params: CheckinInput) -> ServiceData:
        """
        The service checkins entire structures under the roots and the parameters given.
        """
        return cls.execute_soa_method(
            method_name='checkinForProcessSimulate',
            library='Internal-Manufacturing',
            service_date='2008_12',
            service_name='Core',
            params={'rootLines': rootLines, 'params': params},
            response_cls=ServiceData,
        )

    @classmethod
    def checkoutForProcessSimulate(cls, rootLines: List[BusinessObject], params: CheckoutInput) -> ServiceData:
        """
        The service checkouts entire structures under the roots and the parameters given.
        """
        return cls.execute_soa_method(
            method_name='checkoutForProcessSimulate',
            library='Internal-Manufacturing',
            service_date='2008_12',
            service_name='Core',
            params={'rootLines': rootLines, 'params': params},
            response_cls=ServiceData,
        )



from dataclasses import field, dataclass
from typing import Dict, List, Optional

from tcsoa.config import TcSoaConfig


class TcBaseObj(object):
    def __getitem__(self, item):
        """ Makes the object subscriptable - you may use dict index access """
        return getattr(self, item)


@dataclass
class TcBoClass(TcBaseObj):
    uid: str = ''
    objectID: str = ''
    type: str = ''
    className: str = ''
    props: Dict[str, Dict[str, List[str]]] = field(default_factory=dict, hash=False, compare=False)

    def prop(self, prop_name, db_value=False) -> List[Optional[str]]:
        if TcSoaConfig.global_obj_cache_enabled:
            prop = TcSoaConfig.internal_get_prop(self.uid, prop_name)
        else:
            prop = None
            if self.props and prop_name in self.props:
                prop = self.props[prop_name]
        if prop:
            attr = 'dbValues' if db_value else 'uiValues'
            if attr in prop:
                if 'isNulls' in prop:
                    return [None if prop['isNulls'][idx] else val for idx, val in enumerate(prop[attr])]
                return prop[attr]
        return [None]


class TcService:
    @classmethod
    def execute_soa_method(cls, method_name, library, service_date, service_name, params, response_cls):
        service_id = f'{library}-{service_date.replace("_", "-")}-{service_name}'
        return TcSoaConfig.backend.execute(service_id, method_name, params, response_cls)



from __future__ import annotations

from tcsoa.gen.Internal.Core._2018_11.LogicalObject import AddIncludedLogicalObjectsResponse, AddIncludedLOInput
from tcsoa.gen.Internal.Core._2018_11.FileManagement import GetTransientTicketsDownloadResponse, GetTransientTicketsDownloadInput
from typing import List
from tcsoa.base import TcService


class FileManagementService(TcService):

    @classmethod
    def getTransientFileTicketsForDownload(cls, tickets: List[GetTransientTicketsDownloadInput]) -> GetTransientTicketsDownloadResponse:
        """
        This operation obtains File Management System (FMS) transient file read tickets for the supplied transient file
        write tickets. Optionally it deletes the file from temporary storage if specified in input.
        
        Use cases:
        One SOA client want to send another SOA client a temporary file using the FMS system. The two clients already
        have means of communicating with each other without using SOA but want to use the FMS system for bulk
        transfers. 
        
        The client sending the file to the other client uses the following two  service operations:
        Teamcenter::Services::Core::FilemanagementService::getTransientFileTicketsForUpload to generate a write ticket
        for upload, followed by Teamcenter::Soa::Client::FccProxy::uploadFiles to upload the file using the write
        ticket.
        
        In order to give the other SOA client a read ticket for this uploaded file, a new ticket needs to be generated.
        The client sending the file then uses this operation getTransientFileTicketsForDownload to generate a read
        ticket for the file that was just uploaded.
        
        After the other SOA client receives this ticket, it uses the
        Teamcenter::Soa::Client::FccProxy::downloadFilesToLocation operation to download the file using the read ticket.
        """
        return cls.execute_soa_method(
            method_name='getTransientFileTicketsForDownload',
            library='Internal-Core',
            service_date='2018_11',
            service_name='FileManagement',
            params={'tickets': tickets},
            response_cls=GetTransientTicketsDownloadResponse,
        )


class LogicalObjectService(TcService):

    @classmethod
    def addIncludedLogicalObjects(cls, addIncludedLOInput: AddIncludedLOInput) -> AddIncludedLogicalObjectsResponse:
        """
        This operation adds other logical object types to an existing Logical Object Type based on the specified input.
        
        Use cases:
        This operation is invoked to add included logical objects to an existing Logical Object Type.
        """
        return cls.execute_soa_method(
            method_name='addIncludedLogicalObjects',
            library='Internal-Core',
            service_date='2018_11',
            service_name='LogicalObject',
            params={'addIncludedLOInput': addIncludedLOInput},
            response_cls=AddIncludedLogicalObjectsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetActivityTimesResponse(TcBaseObj):
    """
    The response structure of the getActivityTimes operation.
    
    :var results: A vector of GetActivityTimesResult structures that holds for each root node the list of activity
    times indexed by category.
    :var serviceData: This is a common data structure that is used to return Teamcenter model data from a service
    request. This member also holds service exceptions.
    """
    results: List[GetActivityTimesResult] = ()
    serviceData: ServiceData = None


@dataclass
class GetActivityTimesResult(TcBaseObj):
    """
    A structure that contains for a specific root node passed to the getActivityTimes operation the map of accumulated
    activity times per category.
    
    :var rootNode: The root of the process or operation tree for which the activity times have been calculated.
    :var timePerCategoryMap: Contains the computed activity time for each category.
    """
    rootNode: BusinessObject = None
    timePerCategoryMap: ActivityTimePerCategoryMap = None


@dataclass
class CalculateCriticalPathResponseEx(TcBaseObj):
    """
    The response structure for the calculateCriticalPathEx operation. If it is not possible to compute the critical
    path for a specific root object, no entry will be added  to the results vector. Instead the serviceData member will
    contain an description of the failure cause.
    
    :var results: A list of CalculateCriticalPathResultEx structures for each path returned.
    :var serviceData: Contains a list of error descriptions if the path computation has failed.
    """
    results: List[CalculateCriticalPathResultEx] = ()
    serviceData: ServiceData = None


@dataclass
class CalculateCriticalPathResultEx(TcBaseObj):
    """
    A structure that represents a path returned by the computeCriticalPathEx operation.
    
    :var object: the root object for which this path was computed
    :var components: A list of components that make up the path. This includes also the connecting flows.
    :var duration: The total length of the path.
    """
    object: BusinessObject = None
    components: List[BusinessObject] = ()
    duration: float = 0.0


"""
A map that contains activity time values for different activity categories.
"""
ActivityTimePerCategoryMap = Dict[str, float]



from __future__ import annotations

from typing import List
from tcsoa.base import TcService


class CodeCoverageManagementService(TcService):

    @classmethod
    def performOperation(cls, opType: str, arguments: List[str]) -> str:
        """
        Service for CodeCoverage integration. This service allows users to start/stop the tcserver code coverage hit
        countering process from the RAC UI.
        
        Use cases:
        From the RAC UI, before users start a test to collect test-to-SOA interface information, the user can click to
        start button to start the tcserver code coverage countering process. And after the test completed, users can
        click stop button and stop the tcserver code coverage countering process, and the hit information will be saved
        to a local file specified by users.
        """
        return cls.execute_soa_method(
            method_name='performOperation',
            library='Internal-CodeCoverage',
            service_date='2011_06',
            service_name='CodeCoverageManagement',
            params={'opType': opType, 'arguments': arguments},
            response_cls=str,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class FindAndRepopulateDynamicIPAsResponse(TcBaseObj):
    """
    The response of findAndRepopulateDynamicIPAs operation.
    
    It includes:
    
    1. The response includes data about the dynamic IPAs of all the given bop line objects. For each dynamic IPA, the
    response includes its content (i.e. parts underneath). There can be several dynamic IPAs for every process.
    This data is returned for every given bop line, no matter whether its dynamic IPAs were originally empty or not.
    
    2. The following partial errors are returned as part of the serviceData in case invalid parameters are passed to
    the operation:-
    
        25439 :- The type of the given object is not supported
        25440 :- The given object doesn&apos;t have any dynamic IPA.
    
    :var dynamicIPAsData: A list of DynamicIPAsOfLine. Each element in the list contains data about all the dynamic
    IPAs of a single line.
    :var serviceData: The service data.
    
    The following partial errors are returned in case invalid parameters are passed to the operation:-
    
    25439 :- The type of the given object is not supported.
    
    25440 :- The given object doesn&apos;t have any dynamic IPA.
    """
    dynamicIPAsData: List[DynamicIPAsOfLine] = ()
    serviceData: ServiceData = None


@dataclass
class RepopulateDynamicIPAsData(TcBaseObj):
    """
    Input structure for the input for repopulateDynamicIPAs service.
    
    :var topLine: The top BOPLine of the window.
    :var ids: A list of absolute occurrence IDs of processes (Mfg0BvrProcess) or studies (Mfg0BvrShdStudy) in context
    of topLine.
    """
    topLine: BusinessObject = None
    ids: List[str] = ()


@dataclass
class RepopulateDynamicIPAsResponse(TcBaseObj):
    """
    Response structure for repopulateDynamicIPAs service.
    
    :var dynamicIPAsData: List of data containing information about process lines (Mfg0BvrProcess) and their respective
    Dynamic IPA nodes (Mfg0BvrDynamicIPA).
    :var serviceData: Standard service data.
    """
    dynamicIPAsData: List[DynamicIPAsProcLineInfo] = ()
    serviceData: ServiceData = None


@dataclass
class ConsumedPartsInfo(TcBaseObj):
    """
    Structure containing information related to consumed parts.
    
    :var consumedPart: A BOPLine of type Mfg0BvrPart present in the process structure.
    :var referencedPart: A BOMLine of type Mfg0BvrPart present in the product window related to consumedPart.
    :var occInformation: A map ( string, string ) of property information related to the occurrence of consumedPart.
    """
    consumedPart: BusinessObject = None
    referencedPart: BusinessObject = None
    occInformation: OccInformation = None


@dataclass
class DynamicIPAData(TcBaseObj):
    """
    A structure that contains data about a single dynamic IPA.
    
    :var dynamicIPA: A business object from type Mfg0BvrDynamicIPA.
    :var consumedParts: A list of of BOMLine of type Mfg0BvrPart. The parts are the content (the direct children) of
    the dynamic IPA.
    """
    dynamicIPA: BusinessObject = None
    consumedParts: List[BusinessObject] = ()


@dataclass
class DynamicIPAInfo(TcBaseObj):
    """
    Structure containing information related to dynamic IPA node.
    
    :var dynamicIPA: Dynamic IPA node (Mfg0BvrDynamicIPA).
    :var consumedParts: A list of child parts of dynamic IPA node and its related information.
    :var occInformation: A map ( string, string ) of property information related to the of dynamic IPA node.
    """
    dynamicIPA: BusinessObject = None
    consumedParts: List[ConsumedPartsInfo] = ()
    occInformation: OccInformation = None


@dataclass
class DynamicIPAsOfLine(TcBaseObj):
    """
    This structure contains data about all the dynamic IPAs of a single bop line object.
    
    :var bopLine: A business object from type Mfg0BvrProcess or Mfg0BvrShdStudy.
    :var dynamicIPAsData: A list of DynamicIPAData structures. Each element contains data about a single dynamic IPA.
    """
    bopLine: BusinessObject = None
    dynamicIPAsData: List[DynamicIPAData] = ()


@dataclass
class DynamicIPAsProcLineInfo(TcBaseObj):
    """
    This structure contains data about all the dynamic IPAs of a single bop line object.
    
    :var bopLine: A business object of type Mfg0BvrProcess or Mfg0BvrShdStudy.
    :var dynamicIPAsInfo: A list of Dynamic IPA nodes (Mfg0BvrDynamicIPA) and its related information.
    """
    bopLine: BusinessObject = None
    dynamicIPAsInfo: List[DynamicIPAInfo] = ()


"""
A map ( string ,string ) of property information related to current occurrence.    
"""
OccInformation = Dict[str, str]



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from tcsoa.gen.BusinessObjects import BOMLine, BusinessObject, StructureContext, WorkspaceObject
from dataclasses import dataclass


@dataclass
class GetStructureContextLinesResponse(TcBaseObj):
    """
    Get toplines from the BomWindow setup by StructureContext and any selected bomlines.
    
    :var topLines: Map of StructureContext to its toplines.
    For example, Product Structure and Process Structure top lines.
    :var selectedLines: Map of StructureContext to any selected child lines.
    Note: If the topline is selected this map will be empty.
    :var serviceData: Structure to capture any partial errors.
    """
    topLines: StructureContextToLinesMap2 = None
    selectedLines: StructureContextToLinesMap2 = None
    serviceData: ServiceData = None


@dataclass
class PasteDuplicateStructureResponse(TcBaseObj):
    """
    response structure of operation that clones the src objects before pasting.
    
    :var createdICRevs: any newly created Incremental Change Revisions.
    :var createdFutureICRevs: any newly created future IC revs
    :var serviceData: service data returns the populated targetLines along with any partial errors.
    :var newChildLines: the map of targetline and the newly created lines under it.
    """
    createdICRevs: List[WorkspaceObject] = ()
    createdFutureICRevs: List[WorkspaceObject] = ()
    serviceData: ServiceData = None
    newChildLines: LineToLinesMap = None


@dataclass
class CopyEBOPStructureResponse(TcBaseObj):
    """
    response structure for the service method copyEBOPStructure.
    
    :var createdICRevs: any newly created incremental change revisions
    :var createdFutureICRevs: any newly created future Incremental Change Revisions.
    :var serviceData: the updated item(root) and any partial errors returned here.
    :var updatedObject: The updatedObject which is the itemrevision of the passed in item.
    """
    createdICRevs: List[WorkspaceObject] = ()
    createdFutureICRevs: List[WorkspaceObject] = ()
    serviceData: ServiceData = None
    updatedObject: WorkspaceObject = None


"""
Map of a line to it's new pasted child lines
"""
LineToLinesMap = Dict[BusinessObject, List[BusinessObject]]


"""
map of StructureContexts to its associated BomLines
"""
StructureContextToLinesMap2 = Dict[StructureContext, List[BOMLine]]



from __future__ import annotations

from tcsoa.gen.Cad._2007_06.StructureManagement import CreateUpdateClassicOptionsInput, DelClassicOptionsInput, DeleteVariantCondInput, GetConfiguredItemRevisionResponse, CreateOrUpdateVariantCondInput, GetConfiguredItemRevisionInfo
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class StructureManagementService(TcService):

    @classmethod
    def createOrUpdateClassicOptions(cls, inputObjects: List[CreateUpdateClassicOptionsInput]) -> ServiceData:
        """
        In the Create mode this operation creates a new option(s), with given option values, and declares them against
        the given ItemRevision object. In the update mode following operations can be performed with the given option 
        - 1.    Replace the current text value for the specified index with a new string from option revision. 
        - 2.    Add a new value to the option revision. 
        - 3.    Remove an existing value from the option revision.
        
        
        
        Use cases:
        This operation will be used when user wants to create classic variant options for a given BOMLine object(s).
        This also can be used to update an Option 
        - a) adding a new value 
        - b) removing an existing value 
        - c) replace an existing value by new value.
        
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateClassicOptions',
            library='Cad',
            service_date='2007_06',
            service_name='StructureManagement',
            params={'inputObjects': inputObjects},
            response_cls=ServiceData,
        )

    @classmethod
    def createOrUpdateVariantConditions(cls, inputObjects: List[CreateOrUpdateVariantCondInput]) -> ServiceData:
        """
        This operation is to 'create' or 'update' (depending on the Operation) a variantCondition ( which is variant
        expression of type load if) for a BOMLine object.
        
        Use cases:
        This operation will be used when user wants to create a new or update an existing classic variant condition for
        a given BOMLine objects.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateVariantConditions',
            library='Cad',
            service_date='2007_06',
            service_name='StructureManagement',
            params={'inputObjects': inputObjects},
            response_cls=ServiceData,
        )

    @classmethod
    def deleteClassicOptions(cls, inputObjects: List[DelClassicOptionsInput]) -> ServiceData:
        """
        Delete option deletes the option and all the values associated with it.
        """
        return cls.execute_soa_method(
            method_name='deleteClassicOptions',
            library='Cad',
            service_date='2007_06',
            service_name='StructureManagement',
            params={'inputObjects': inputObjects},
            response_cls=ServiceData,
        )

    @classmethod
    def deleteVariantConditions(cls, inputObjects: List[DeleteVariantCondInput]) -> ServiceData:
        """
        This service will be used to delete the variant Condition(load_if) associated with a BOMLine If the variant
        condition exists then it will be deleted.
        Failure will be with client id and error message in the ServiceData.
        """
        return cls.execute_soa_method(
            method_name='deleteVariantConditions',
            library='Cad',
            service_date='2007_06',
            service_name='StructureManagement',
            params={'inputObjects': inputObjects},
            response_cls=ServiceData,
        )

    @classmethod
    def getConfiguredItemRevision(cls, inputs: List[GetConfiguredItemRevisionInfo]) -> GetConfiguredItemRevisionResponse:
        """
        Finds the revision of the given item / item revision that is configured when the given revision rule is used to
        configure the given item / item revision.
        """
        return cls.execute_soa_method(
            method_name='getConfiguredItemRevision',
            library='Cad',
            service_date='2007_06',
            service_name='StructureManagement',
            params={'inputs': inputs},
            response_cls=GetConfiguredItemRevisionResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Manufacturing._2013_05.IPAManagement import FileTicketDetails
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from typing import Dict, List


@dataclass
class SaveDynamicIPALinesResponse(TcBaseObj):
    """
    FMS file ticket of the log file for input dynamicIPA lines. The following partial errors may be returned:
    - 253126 : The target object cannot be modified, because the user does not have write access.
    
    
    
    :var serviceData: Partial errors as part of the serviceData. These errors will be those encountered during various
    aspects of traversal of process structure.
    :var logFile: The details of the log file ticket.
    """
    serviceData: ServiceData = None
    logFile: FileTicketDetails = None


@dataclass
class CleanDynamicIPALinesResponse(TcBaseObj):
    """
    A FMS file ticket for the log file for the process and the following partial errors may be returned:
    
    253130 : The Clean Dynamic Assembly Tree operation has failed. Please check the Teamcenter server syslog file for
    more information.
    253134 : The Clean Dynamic Assembly Tree operation has failed, because there are no Dynamic Assembly Tree nodes
    related to the current configuring structure.
    253131 : The Create/Update Dynamic Assembly Tree operation has failed. Please check the Teamcenter server syslog
    file for more information.
    200411    - The dynamic In Process Assembly tree cannot be found.
    
    :var serviceData: Partial errors as part of the serviceData.
    :var logFileTicket: The details of the log file ticket.
    """
    serviceData: ServiceData = None
    logFileTicket: FileTicketDetails = None


@dataclass
class DynamicIPAInputInfo(TcBaseObj):
    """
    BOP line along with its recursive flag information.
    
    :var inputLine: Business objects representing BOP line.
    :var isRecursive: Flag to consider processes in sub-hierarchy to clean the dynamic IPA lines.
    
    true - Cleans the dyanmic IPA lines for all the processes in the sub-hierarchy of the input BOPLines.
    false- Cleans the Dynamic IPA lines for the input BOPLines only.
    """
    inputLine: BusinessObject = None
    isRecursive: bool = False


@dataclass
class DynamicIPALinesResponse(TcBaseObj):
    """
    A map of input BOPLines and their corresonding  DIPA(Mfg0BvrDynamicIPA) nodes and a FMS file ticket for the log
    file for the process. The following partial errors may be returned:
    
    253131 : The Create/Update Dynamic Assembly Tree operation has failed. Please check the Teamcenter server syslog
    file for more information.
    253132 : The Create/Update Dynamic Assembly Tree operation has failed, because there are more than one configuring
    structures. Please check the Teamcenter server syslog file for more information.
    253133 : The Create/Update Dynamic Assembly Tree operation has failed, because the selected process(es) do not have
    predecessors.
    
    :var bopLineToIPALinesMap: A map (BOMLine, list of BOMLline) where the key represents BOPLine of type
    Mfg0BvrProcess and value is list of BOPLine of type Mfg0BvrDynamicIPA.
    :var serviceData: Partial errors as part of the serviceData.
    :var logFileTicket: The details of the log file ticket.
    """
    bopLineToIPALinesMap: BopLineToDynamicIPALinesMap = None
    serviceData: ServiceData = None
    logFileTicket: FileTicketDetails = None


"""
A map (BOMLine, list of BOMLline) where the key represents BOPLine of type Mfg0BvrProcess and value is list of BOPLine of type Mfg0BvrDynamicIPA.
"""
BopLineToDynamicIPALinesMap = Dict[BusinessObject, List[BusinessObject]]



from __future__ import annotations

from tcsoa.gen.Internal.AWS2._2016_03.Finder import GetClassificationPropsResponse, SearchResponse, SaveColumnConfigData, SearchInput, ColumnConfigInput
from tcsoa.gen.Internal.AWS2._2016_03.DataManagement import GetUnprocessedXRTResponse, GetStyleSheetResponse, DSInfo
from tcsoa.gen.Internal.AWS2._2016_03.RequirementsManagement import ExportToApplicationInputData2, ExportToApplicationResponse2
from tcsoa.gen.Internal.AWS2._2016_03.UiConfig import GetVisibleCommandsResponse, SaveColumnConfigData, GetVisibleCommandsInfo, GetOrResetUIColumnConfigInput, GetOrResetUIColumnConfigResponse
from tcsoa.gen.Internal.AWS2._2016_04.DataManagement import GetStyleSheetIn
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService
from tcsoa.gen.BusinessObjects import WorkspaceObject


class UiConfigService(TcService):

    @classmethod
    def getOrResetUIColumnConfigs(cls, getOrResetUiConfigsIn: List[GetOrResetUIColumnConfigInput]) -> GetOrResetUIColumnConfigResponse:
        """
        This operation returns information used by the client to render the User Interface. The information returned
        includes column configuration information. 
        If the resetColumnConfig flag is "True", this operation deletes the column configuration of the input scope and
        then queries the new effective UI column configuration for the login user. This operation will only clear the
        login user column configurations.
        
        Use cases:
        Request UI Configuration(s) based on the current login user
        Client requests the column  information for one or more client scopes using this operation and scope as login
        user.
        
        Request UI Configuration(s) based on a specific Teamcenter scope
        Client requests the column information for one or more client scopes using this operation and scope as Role,
        Site or Group.
        
        Request to reset UI Column Configuration(s) based on the current login user
        If a client needs to reset the column information for login user scope, they can use this operation. The new
        effective UI column configuration will be retrieved for the login user.
        """
        return cls.execute_soa_method(
            method_name='getOrResetUIColumnConfigs',
            library='Internal-AWS2',
            service_date='2016_03',
            service_name='UiConfig',
            params={'getOrResetUiConfigsIn': getOrResetUiConfigsIn},
            response_cls=GetOrResetUIColumnConfigResponse,
        )

    @classmethod
    def getVisibleCommands(cls, getVisibleCommandsInfo: List[GetVisibleCommandsInfo]) -> GetVisibleCommandsResponse:
        """
        This operation returns information used by the client to render the User Interface. The information returned
        includes commands that are visible.
        """
        return cls.execute_soa_method(
            method_name='getVisibleCommands',
            library='Internal-AWS2',
            service_date='2016_03',
            service_name='UiConfig',
            params={'getVisibleCommandsInfo': getVisibleCommandsInfo},
            response_cls=GetVisibleCommandsResponse,
        )

    @classmethod
    def saveUIColumnConfigs(cls, columnConfigurations: List[SaveColumnConfigData]) -> ServiceData:
        """
        This service operation saves column configuration information to the Teamcenter database. A Teamcenter client
        may use this information to render the user interface as the user navigates the Teamcenter client. Default
        column configurations are initally created by an utility during install. This operation will modify the default
        column configuration for the site, group, role or a specific user.
        
        Use cases:
        User level column configuration:
        A User wants to change the default column configuration for a Client Scope when he is logged into the client.
        The saved configuration will override the default configuration for the user's applicable role, group or site.
        """
        return cls.execute_soa_method(
            method_name='saveUIColumnConfigs',
            library='Internal-AWS2',
            service_date='2016_03',
            service_name='UiConfig',
            params={'columnConfigurations': columnConfigurations},
            response_cls=ServiceData,
        )


class DataManagementService(TcService):

    @classmethod
    def getStyleSheet2(cls, processEntireXRT: bool, input: List[GetStyleSheetIn]) -> GetStyleSheetResponse:
        """
        This operation returns the stylesheets and the required data to present that stylesheet for each input object.
        
        Use cases:
        Use Case 1: Open an object in ActiveWorkspace.
        When an object is selected in ActiveWorkspace, the user may choose to invoke the open operation. When that
        operation is executed the getStylesheet call is invoked and the Stylesheet is processed in order to display the
        data to the user.
        
        Use Case 2: List view with Summary
        When an object is selected in the ActiveWorkspace navigator, when in list view with summary mode, the
        getStyleSheet SOA operation is invoked, and the returned data is parsed/processed in order to present the data
        to the user in the Summary panel.
        
        Use Case 3: Show Object Info
        When an object is selected in ActiveWorkspace, a user may choose the "Show Object Info" command.  When
        executed, a panel slides out from the right hand side of the application, and the selected objects data is
        presented.  In order to populate the panel, the getStylesheet call is invoked and the returned data is parsed
        and processed in order to build the UI.
        """
        return cls.execute_soa_method(
            method_name='getStyleSheet2',
            library='Internal-AWS2',
            service_date='2016_03',
            service_name='DataManagement',
            params={'processEntireXRT': processEntireXRT, 'input': input},
            response_cls=GetStyleSheetResponse,
        )

    @classmethod
    def getUnprocessedXRT(cls, client: str, type: str, location: str, sublocation: str, stylesheetType: str, preferenceLocation: str) -> GetUnprocessedXRTResponse:
        """
        This operation returns the unprocessed XRT stylesheet.
        
        Use cases:
        XRT administration using the internal client "xrtFiddle".
        """
        return cls.execute_soa_method(
            method_name='getUnprocessedXRT',
            library='Internal-AWS2',
            service_date='2016_03',
            service_name='DataManagement',
            params={'client': client, 'type': type, 'location': location, 'sublocation': sublocation, 'stylesheetType': stylesheetType, 'preferenceLocation': preferenceLocation},
            response_cls=GetUnprocessedXRTResponse,
        )

    @classmethod
    def saveXRT(cls, dsInfo: DSInfo) -> ServiceData:
        """
        Saves an XRT definition to the specified context.   In this version, saving of injected datasets is not
        supported.
        
        Use cases:
        Save an XRT from the internal client called xrtFiddle.
        """
        return cls.execute_soa_method(
            method_name='saveXRT',
            library='Internal-AWS2',
            service_date='2016_03',
            service_name='DataManagement',
            params={'dsInfo': dsInfo},
            response_cls=ServiceData,
        )


class FinderService(TcService):

    @classmethod
    def performSearch(cls, searchInput: SearchInput, columnConfigInput: ColumnConfigInput, saveColumnConfigData: SaveColumnConfigData) -> SearchResponse:
        """
        This operation routes a search request to a specific provider specified as providerName in the searchInput
        structures
        
        The framework allows a custom solution to be able to provide a new specific provider to collect data, perform
        sorting and filtering. The provider can be a User Inbox retriever or Full Text searcher, for example. The new
        data provider can be encapsulated via a new runtime business object from Fnd0BaseProvider class. The
        implementation is done using its fnd0performSearch operation.
        
        RuntimeBusinessObject
        ---- Fnd0BaseProvider (foundation template)
        -------- Fnd0GetChildrenProvider (foundation template)
        -------- Awp0FullTextSearchProvider (aws template)
        -------- Awp0TaskSearchProvider (aws template)
        -------- Aos0TestProvider (aosinternal template)
        -------- etc.
        This operation provides a common interface to send the request to and receive the response from a new data
        provider. Ultimately it allows common framework in UI to support filter, pagination, and sorting.
        
        This operation allows the caller to send the search input, filter values, and sorting data. These input values
        are then to be passed to the fnd0performSearch operation on the server to collect, sort, and filter its results.
        
        The first two input parameters are important. The first input is the provider name. This is a string to
        represent the type name of RuntimeBusinessObject which this request should be routed to. If the template that
        contains the class is not installed, a partial error 217016 is returned. The second input is the search input
        map. The key is different per each provider. For example, for Full Text searcher, the input key would be a
        searchString. For User Inbox retriever, the input key would be an Inbox Type. The fnd0performSearch
        implementation for each provider takes into account the key name as it is used to store the values in
        OperationInput object. 
        
        The internal property name is passed as part of search input to allow grouping of business objects. The
        grouping allows for added functionality such as color coding, etc. If no internal property name is passed,
        default value is assumed as "object_type" to group the business objects.
        
        If new list of column config data is passed in to be saved, it will get saved in the database for the login
        user scope. This will be done prior to performing the search. 
        
        The column configuration input has the criteria needed to query for the columns configured for the provided
        client and client scope URI. The final list of columns are identified based on the types returned back in the
        search results and the operation type( Union / Intersection / Configured ) configured for the client scope URI.
        Finally the determined column attributes and the attributes to be inflated passed in as part of search input
        criteria are combined. This combined list of attributes along with the business objects found during search are
        added to the Service data.
        
        Use cases:
        Use Case 1: User clicks on Inbox tile
        A search request will be made by passing in the needed criteria. In this case the the provider name is
        "Awp0InboxProvider". It queries for the data and finds the effective columns for the client scope URI passed in
        through column config input. The results are finally shown in the view type selected in client. If table view
        is selected the returned effective columns will be shown.
        
        Use Case 2: User performs search by entering a keyword in global search input box.
        A search request will be made by passing in the needed criteria that includes the keyword user has typed in. In
        this case the the provider name is "Awp0FullTextSearchProvider". It queries for the data and finds the
        effective columns for the client scope URI passed in through column config input. The results are finally shown
        in the view type selected in client. If table view is selected the returned effective columns will be shown.
        
        Use Case 3: User tries to save new column configuartions after performing search.
        The new list of column configurations will be saved for the login user scope. Later the search request will be
        made by passing in the needed criteria that includes the keyword user had typed in. In this case the the
        provider name is "Awp0FullTextSearchProvider". It queries for the data and finds the effective columns for the
        client scope URI passed in through column config input. The results are finally shown in the view type selected
        in client. If table view is selected the returned effective columns will be shown.
        """
        return cls.execute_soa_method(
            method_name='performSearch',
            library='Internal-AWS2',
            service_date='2016_03',
            service_name='Finder',
            params={'searchInput': searchInput, 'columnConfigInput': columnConfigInput, 'saveColumnConfigData': saveColumnConfigData},
            response_cls=SearchResponse,
        )

    @classmethod
    def getClassificationProps(cls, objects: List[WorkspaceObject], showAllProperties: bool) -> GetClassificationPropsResponse:
        """
        This operation provides classification properties for a given WorkspaceObject(s).
        
        Use cases:
        User wants to find classification properties for WorkspaceObjects. Each time a WorkspaceObject is classified in
        a classification class a classification object (icm0) is created. After searching for all the classification
        objects corresponding to a workspace object, user can find more information about the classification(s) of a
        workspace object. The operation getClassificationProps() can be used to get detailed information about the
        classification objects.
        """
        return cls.execute_soa_method(
            method_name='getClassificationProps',
            library='Internal-AWS2',
            service_date='2016_03',
            service_name='Finder',
            params={'objects': objects, 'showAllProperties': showAllProperties},
            response_cls=GetClassificationPropsResponse,
        )


class RequirementsManagementService(TcService):

    @classmethod
    def exportToApplication(cls, input: List[ExportToApplicationInputData2]) -> ExportToApplicationResponse2:
        """
        This operation will exports the selected occurrences in the structure to the application such as MSWord and
        MSExcel. The behavior of export operation is controlled by applicationFormat argument specified in
        ExportToApplicationInput2 structure.
        Following are supported application formats from input.
        
        - MSExcelLive - This mode provides the capability of editing of properties of  underlying ItemRevision  in
        MSExcel and save the changes to Teamcenter. The generated sheet allows user to login and change the properties
        from MSExcel application. The changes will be saved to Teamcenter when user exits out of the cell in MSExcel
        application.  Teamcenter extensions for Microsoft Office should be installed on the client machine to support
        this application format.
        - MSWordXMLLive - This mode provides the capability of structural editing of SpecElementRevisions in MSWord.
        Users can edit the rich text of Requirement and perform structural changes to the exported Requirement(s) in
        the word document. Office Client should be installed on the client machine to support this application format.
        - MSExcel - This mode provides the capability of exporting objects to static MSExcel application. 
        - MSExcelReimport- This mode provides the capability of exporting objects to MSExcel application for reimport
        purpose. The generated sheet can be used to for add/remove structure elements or to update the properties on
        the SpecElement revision. The generated sheet should be reimported back to  Teamcenter. Office Client should be
        installed on the client machine to support this application format.
        - MSWordCompare - This mode provides the capability of comparing the contents of source and target objects
        using MSWord application.  The comparison document is generated asynchonously in the background and user is
        notified when the comparison document is generated. Dispatcher and notification services should be running to
        support this application format.
        - HTML- This mode provides the capability of fetching the HTML contents  of input SpecElement objects.
        
        
        
        Use cases:
        - User loads a structure in ACE (Active Content Experience) panel. The ACE panel is used for smart navigation
        of occurrences in a structure. User selects an occurrence and clicks on the Export To Word or Export to Excel
        button in the toolbar. This action will allow users to select a template that will be used for export purpose.
        If user has selected a single occurrence and clicks on export, then the occurrence and all its children are
        exported to MSWord or MSExcel. If user has selected multiple occurrences and clicks on export, then the
        selected occurrences are exported to MSWord or MSExcel.
        - User selects single or multiple objects in Active Workspace Client. User clicks on the Export to Excel button
        in the toolbar. User can select View, Edit or Reimport option. Depending upon the option selected, a static
        MSExcel report or a Live MSExcel sheet or a sheet is generated that can be reimported back to Teamcenter. 
        - User select s single or multiple objects in Active Workspace Client and sets them as source objects to
        compare. Similarly user selects single or multiple target objects to compare. User clicks on the Export to Word
        button in the toolbar along with the checkbox for compare contents. User selects the compare contents checkbox
        that allows user to compare the MSWord contents of source and target objects. User is notified when the
        comparison document is generated and attached to a dataset.
        - User selects a SpecElement created from Active Workspace client and clicks on the Viewer tab. The HTML
        contents of the selected SpecElement is shown in the Viewer tab.
        
        """
        return cls.execute_soa_method(
            method_name='exportToApplication',
            library='Internal-AWS2',
            service_date='2016_03',
            service_name='RequirementsManagement',
            params={'input': input},
            response_cls=ExportToApplicationResponse2,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Manufacturing._2015_10.StructureManagement import AdditionalInfo
from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class FindBrokenPartsDetails(TcBaseObj):
    """
    FindBrokenPartsDetails represents a list of first level children Clone Stable ID (CSID), CSID to property map, CSID
    children map for broken parts for each product view Dataset.
    
    :var targetPV: Target Dataset for finding broken parts.
    :var firstLevelChildrenCSID: A list of clone stable ID (CSID) for the first level children.
    :var propertyInfo: Properties for a CSID.
    :var childrenMap: A map (string, list of strings) representing parent and children&rsquo;s CSID, where key
    represents CSID of parent and value represents the list of CSID values of the children.
    :var additonalInfo: Currently unused, Possible future example usage:
    additionalInfo.strToIntVectorMap["hasIPANodes"] = (1);
    """
    targetPV: BusinessObject = None
    firstLevelChildrenCSID: List[str] = ()
    propertyInfo: BrokenPartsCSIDToPropsInfo = None
    childrenMap: BrokenPartsParentToChildrenMap = None
    additonalInfo: AdditionalInfo = None


@dataclass
class FindBrokenPartsInPVInputInfo(TcBaseObj):
    """
    This input structure contains target product view Dataset objects and target scope (BOMLine) for identifying the
    broken parts and any optional key/value pairs of information.
    
    :var targetDatasets: A list of Product View Dataset objects.
    :var targetScope: Scope BOMLine for resolving the Product View.
    :var additionalInfo: Currently unused. Possible future example usage:
    additionalInfo.strToIntVectorMap["ignoreIPA"] = (1);
    """
    targetDatasets: List[BusinessObject] = ()
    targetScope: BusinessObject = None
    additionalInfo: AdditionalInfo = None


@dataclass
class FindBrokenPartsInPVResponse(TcBaseObj):
    """
    FindBrokenPartsInPVResponse represents list of FindBrokenPartsDetails and any partial errors captured during the
    service call.
    
    :var serviceData: Object that captures any partial errors.
    :var brokenPartsDetail: A list of broken parts details for input Dataset objects.
    """
    serviceData: ServiceData = None
    brokenPartsDetail: List[FindBrokenPartsDetails] = ()


@dataclass
class FindBrokenProductViewsDetails(TcBaseObj):
    """
    FindBrokenProductViewsDetails represents a list of BrokenProductViewsData for each scope.
    
    :var scope: Input BOMLine object.
    :var brokenProductViews: A list of broken product views details for current scope.
    """
    scope: BusinessObject = None
    brokenProductViews: List[BrokenProductViewsData] = ()


@dataclass
class FindBrokenProductViewsInputInfo(TcBaseObj):
    """
    A structure containing list of non-nested scopes for finding broken PVs in their hierarchy and any optional
    key/value pairs of information.
    
    :var unnestedScopes: A list of non-nested BOMLine objects as scope.
    :var additionalInfo: Currently unused. Future example usage: 
    additionalInfo.strToIntVectorMap["getRelatedStructures"] = (1);
    """
    unnestedScopes: List[BusinessObject] = ()
    additionalInfo: AdditionalInfo = None


@dataclass
class FindBrokenProductViewsResponse(TcBaseObj):
    """
    Response for findBrokenProductViews operation. Containing list of FindBrokenProductViewsDetails and any partial
    errors captured during the service call.
    
    :var serviceData: Object that captures any partial errors.
    :var brokenProductViewsDetails: A list of BrokenProductViews for each non-nested scope.
    """
    serviceData: ServiceData = None
    brokenProductViewsDetails: List[FindBrokenProductViewsDetails] = ()


@dataclass
class OpenContextInfo(TcBaseObj):
    """
    The structure containing the BOMWindow information for BOMLine represented by the link. If the CCObject contains
    more than one window, the object member will only be present in the window which actually contains the BOMLine
    represented by the link.
    Elements:
    - context     The root BOMLine of the window.
    - object     BOMLine represented by the link.
    - views     Any AppearanceGroup objects that are opened as part of the link.
    - structureContext     The StructureContext representing this context or window.
    
    
    
    :var context: The root BOMLine of the window.
    :var object: BOMLine represented by the link.
    :var views: Any AppearanceGroup objects that are opened as part of the link.
    :var structureContext: The StructureContext representing this context or window.
    """
    context: BusinessObject = None
    object: BusinessObject = None
    views: List[BusinessObject] = ()
    structureContext: BusinessObject = None


@dataclass
class BrokenPartsCSIDToPropsInfo(TcBaseObj):
    """
    BrokenPartsCSIDToPropsInfo represents the property for rendering clone stable ID (CSID) on the view. It contains a
    list of property types and their corresponding property values in another list. If a value is null or empty for a
    given property then the entry in the propValues for that property would be null.
    
    :var csidValue: Clone stable ID for broken part.
    :var propTypes: A list of property types.
    :var propValues: A list of property values mapping to the property types.
    """
    csidValue: str = ''
    propTypes: List[str] = ()
    propValues: List[str] = ()


@dataclass
class BrokenProductViewsData(TcBaseObj):
    """
    BrokenProductViewsData represents the product view and the attached line for each product view along with any
    additional information in key/value format.
    
    :var productView: Dataset object for the product view.
    :var attachedToLine: Business object for the BOMLine to which the PV is attached.
    :var additionalInfo: Currently unused. Future example usage: additionalInfo.strToIntVectorMap
    ["containsInProcessAssembly"] = (1);
    """
    productView: BusinessObject = None
    attachedToLine: BusinessObject = None
    additionalInfo: AdditionalInfo = None


@dataclass
class ContextGroup(TcBaseObj):
    """
    The structure containing the contextual BOMLine or persistent BusinessObject represented by the link.
    Elements:
    - context     A list of OpenContextInfo objects representing the structures providing the context for the BOMLine
    represented by the link.
    - persistentObject     Persistent BusinessObject represented by the link. Will not be set, if the link represents a
    BOMLine. 
    - applicationId     Currently unused. Probable example usage: applicationId="aws"
    - collaborationContext     If the BOMLine represented by the link is part of structure contained by the CCObject,
    will be set to that object.
    
    
    
    :var contexts: A list of OpenContextInfo objects representing the structures providing the context for the BOMLine
    represented by the link.
    :var persistentObject: Persistent BusinessObject represented by the link. Will not be set, if the link represents a
    BOMLine.
    :var applicationId: Currently unused. Probable example usage: applicationId="aws".
    :var collaborationContext: If the BOMLine represented by the link is part of structure contained by the CCObject,
    will be set to that object.
    """
    contexts: List[OpenContextInfo] = ()
    persistentObject: BusinessObject = None
    applicationId: str = ''
    collaborationContext: BusinessObject = None


@dataclass
class EvaluateLinksData(TcBaseObj):
    """
    A structure containing the link to be evaluated and any optional key/value pairs of information.
    - link     Link to be evaluated.
    - additionalInfo     Currently unused. Probable example usage:  additionalInfo.strToIntVectorMap["GetComponent"]
    =(1)
    
    
    
    :var link: Link to be evaluated.
    :var additionalInfo: additional information to be passed to the client. Currently, one probable value
    strToIntVectorMap["GetComponent"]=(1) is supported. If the first element of the value vector is not 0, the
    operation will return either the CCObject or StructureContext. The objects will be returned in the
    EvaluateLinksResponse additionalInfo.StringToObjectVectorMap member.
    """
    link: str = ''
    additionalInfo: AdditionalInfo = None


@dataclass
class EvaluateLinksResponse(TcBaseObj):
    """
    Response for evaluateLinks operation.
    - group    A list of ContextGroup elements, each representing the BOMLine and it&rsquo;s contextual information
    (CCObject or StructureContext as OpenContextInfo), or persistent BusinessObject.
    - serviceData
    - additionalInfo    Partial errors as part of the serviceData.
    - Currently unused. Probable example usage: additionalInfo.strToObjVectorMap["Component"]=(BOM::12345)
    
    
    
    :var output: A list of ContextGroup elements, each representing the BOMLine and it&rsquo;s contextual information
    (CCObject or StructureContext as OpenContextInfo), or persistent BusinessObject.
    :var serviceData: Partial errors as part of the serviceData.
    :var additionaInfo: If the input has additionalInfo value of "GetComponents" and the link contains a CCObject, then
    additionalInfo will be populated as, additionalInfo.strToObjVectorMap["CC"]=(CCObject), 
    additionalInfo.strToObjVectorMap["CCSC"]=(StructureContext). If the link contains a StructureContext object, the
    data will be returned as additionalInfo.strToObjVectorMap["SC"]=(StructureContext). If the link contains a
    StructureContext representing a Manufacturing Study that is linked to the CCObject, the data will be returned as,
    additionalInfo.strToObjVectorMap["StudySC"]=(StructureContext) in addition to the CCObject.
    """
    output: List[ContextGroup] = ()
    serviceData: ServiceData = None
    additionaInfo: AdditionalInfo = None


"""
BrokenPartsParentToChildrenMap represents the parent to children clone stable ID (CSID) map, where key represents CSID of parent and value represents the list of children&rsquo;s.
"""
BrokenPartsParentToChildrenMap = Dict[str, List[str]]



from __future__ import annotations

from typing import List
from tcsoa.gen.GlobalMultiSite._2007_12.ImportExport import NamesAndValues
from tcsoa.base import TcService
from tcsoa.gen.Internal.GlobalMultiSite._2017_05.ImportExport import TransformDataResponse


class ImportExportService(TcService):

    @classmethod
    def transformData(cls, inputFileTickets: List[str], ruleFileNamesOrTickets: List[str], sessionOptions: List[NamesAndValues]) -> TransformDataResponse:
        """
        This operation transforms a list of input files into a specific output file by following a list of transform
        rules and a group of session options.
        
        Use cases:
        Applications need to transform their files into a file with different formats. The process of the
        transformation needs to be controlled by a group rules and options.
        E.g. TcIF needs to transform TCEngXML to TCXML files.
        """
        return cls.execute_soa_method(
            method_name='transformData',
            library='Internal-GlobalMultiSite',
            service_date='2017_05',
            service_name='ImportExport',
            params={'inputFileTickets': inputFileTickets, 'ruleFileNamesOrTickets': ruleFileNamesOrTickets, 'sessionOptions': sessionOptions},
            response_cls=TransformDataResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, StructureContext, CfgAttachmentLine
from tcsoa.gen.Manufacturing._2009_06.StructureManagement import GetAttachmentLineChildrenResponse, CreateOrUpdateAttachmentsData, GetBOMLineAttachmentsResponse, GetStructureContextActivityLinesResponse, GetStructureContextTopLinesResponse, GetBOMLineActivitiesResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class StructureManagementService(TcService):

    @classmethod
    def getStructureContextActivityLines(cls, scs: List[StructureContext]) -> GetStructureContextActivityLinesResponse:
        """
        Given a vector of StructureContext objects, for each - get the activitylines that are attached to the SC by the
        relation - TC_SC_activities. Currently, this is only created during a population of WorkInstruction page by
        selecting an activity. The following properties are available to the client irrespective of policy:
        al_activity_object_name, al_activity_time
        """
        return cls.execute_soa_method(
            method_name='getStructureContextActivityLines',
            library='Manufacturing',
            service_date='2009_06',
            service_name='StructureManagement',
            params={'scs': scs},
            response_cls=GetStructureContextActivityLinesResponse,
        )

    @classmethod
    def getStructureContextTopLines(cls, scs: List[StructureContext]) -> GetStructureContextTopLinesResponse:
        """
        method to get the toplines of specified StructureContext. Client is responsible for closing any windows that
        are returned during this call. The following properties are available irrespective of policy:bl_line_name
        """
        return cls.execute_soa_method(
            method_name='getStructureContextTopLines',
            library='Manufacturing',
            service_date='2009_06',
            service_name='StructureManagement',
            params={'scs': scs},
            response_cls=GetStructureContextTopLinesResponse,
        )

    @classmethod
    def closeAttachmentWindow(cls, lines: List[BOMLine]) -> ServiceData:
        """
        close any attachment window that got created for the bomline during the soa session. This will only close the
        attachment windows that are created to support the attachment line soa calls.
        """
        return cls.execute_soa_method(
            method_name='closeAttachmentWindow',
            library='Manufacturing',
            service_date='2009_06',
            service_name='StructureManagement',
            params={'lines': lines},
            response_cls=ServiceData,
        )

    @classmethod
    def createOrUpdateAttachments(cls, attachments: List[CreateOrUpdateAttachmentsData]) -> ServiceData:
        """
        create or update attachments. The following properties are loaded automatically for the
        line:me_cl_object_name,me_cl_object_type,me_cl_object_desc and these for the workspaceobject:object_name,
        object_type, object_desc irrespective of policy files.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateAttachments',
            library='Manufacturing',
            service_date='2009_06',
            service_name='StructureManagement',
            params={'attachments': attachments},
            response_cls=ServiceData,
        )

    @classmethod
    def deleteAttachments(cls, lines: List[CfgAttachmentLine]) -> ServiceData:
        """
        remove the specified attachment lines. Only if these lines have a parent is this action performed.
        """
        return cls.execute_soa_method(
            method_name='deleteAttachments',
            library='Manufacturing',
            service_date='2009_06',
            service_name='StructureManagement',
            params={'lines': lines},
            response_cls=ServiceData,
        )

    @classmethod
    def getAttachmentLineChildren(cls, attlines: List[CfgAttachmentLine]) -> GetAttachmentLineChildrenResponse:
        """
        given a vector of input attachmentlines - for each - get the immediate level of child attachment lines.
        For each attachment line the following properties are available on client side automatically:me_cl_object_name,
        me_cl_object_type, me_cl_object_desc
        """
        return cls.execute_soa_method(
            method_name='getAttachmentLineChildren',
            library='Manufacturing',
            service_date='2009_06',
            service_name='StructureManagement',
            params={'attlines': attlines},
            response_cls=GetAttachmentLineChildrenResponse,
        )

    @classmethod
    def getBOMLineActivities(cls, bomLines: List[BOMLine]) -> GetBOMLineActivitiesResponse:
        """
        given a bomline get it's activities (these activities are really the children of the root activity associated
        with the bomline). This assumes that the bomline is  a bopline. The following properties are available on
        client side for each line irrespective of policy: al_activity_object_name, al_activity_time. If the actual
        attachments of these activity lines are desired - use the getProperties method of DataManagementService with
        al_object as the property name.
        """
        return cls.execute_soa_method(
            method_name='getBOMLineActivities',
            library='Manufacturing',
            service_date='2009_06',
            service_name='StructureManagement',
            params={'bomLines': bomLines},
            response_cls=GetBOMLineActivitiesResponse,
        )

    @classmethod
    def getBOMLineAttachments(cls, bomlines: List[BOMLine], filter: List[str]) -> GetBOMLineAttachmentsResponse:
        """
        given a bomline get it's attachments. The follow properties are available on client side irrespective of
        policy: me_cl_object_name, me_cl_object_type, me_cl_object_desc
        """
        return cls.execute_soa_method(
            method_name='getBOMLineAttachments',
            library='Manufacturing',
            service_date='2009_06',
            service_name='StructureManagement',
            params={'bomlines': bomlines, 'filter': filter},
            response_cls=GetBOMLineAttachmentsResponse,
        )



from __future__ import annotations

from tcsoa.gen.Diagramming._2012_09.DiagramManagement import CreateOrUpdateTemplateInputInfo1
from tcsoa.gen.Diagramming._2011_06.DiagramManagement import CreateOrUpdateTemplateResponse
from tcsoa.base import TcService


class DiagramManagementService(TcService):

    @classmethod
    def createOrUpdateTemplate(cls, inputData: CreateOrUpdateTemplateInputInfo1) -> CreateOrUpdateTemplateResponse:
        """
        This operation creates a Diagram Template which is used for creating diagrams for Teamcenter objects. A Diagram
        Template is nothing but a place holder for all the necessary information for creating a diagram. It holds the
        Application Domain which defines how Teamcenter objects will appear on the diagram, Transfer Mode used for
        traversing the given Teamcenter object, the Relation Rule used for finding out the relations between the
        objects to be shown on the diagram, the Stencil or Visio Template which has all the required shapes and the
        Property Map xml file which has the corresponding Teamcenter object mapping.  One can also specify if the
        interface shapes are to be hidden on the diagram created using this template. In such case, even if there are
        interface objects associated with the objects, they are not shown on the diagram.
        
        Use cases:
        Use case 1:
        You can create a diagram template which will be used for creating diagrams. You can provide all the necessary
        information like the Application Domain, the Transfer mode, the Relation Rule, the Stencil/template and the
        property map file and chose if the interface shapes are not to be shown on the diagram. Once user opts to hide
        the shapes, it cannot be modified.
        
        Use case 2: 
        You can select an existing template and edit it. The Description, the Relation Rule can be modified. 
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateTemplate',
            library='Diagramming',
            service_date='2012_09',
            service_name='DiagramManagement',
            params={'inputData': inputData},
            response_cls=CreateOrUpdateTemplateResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Visualization._2010_04.DataManagement import GatherUserAgentDataInfo, GatherInputInfo, GatherServerInfo, GatherResponse, GatherSessionInfo
from typing import List
from tcsoa.base import TcService


class DataManagementService(TcService):

    @classmethod
    def gatherMarkups(cls, selectedInputInfos: List[GatherInputInfo], serverInfo: GatherServerInfo, sessionInfo: GatherSessionInfo, userDataAgentInfo: GatherUserAgentDataInfo) -> GatherResponse:
        """
        This operation finds and retrieves all markup objects related to the object of interest.  It creates a list of
        GatherResponse structure (containing Item uid, Item name, ItemRevision uid, ItemRevision name, Dataset uid,
        Dataset name, Dataset named reference, related markup Dataset uid, related markup Dataset name, related markup
        Dataset named reference, markup tool preferences, related control WorkspaceObject, and additional information
        in form of key value pair strings) based on the list of GatherInputInfo input structure (containing
        WorkspaceObject, related Item, related ItemRevision, related control WorkspaceObject, and additional
        information in the form of key value pair strings), structure of server information GatherServerInfo from where
        the operation is initiated, structure of the session information GatherSessionInfo of the client from where the
        operation is initiated, and structure of client information GatherUserAgentDataInfo from where the operation is
        initiated.
        The required input data from the GatherInputInfo structure is any WorkspaceObject (normally this input is the
        subtype of WorkspaceObject such as Item or Item Revision or Dataset) for which the user wishes to gather all
        the related markup objects.    The input structures for server, session, and client information can be empty.  
        If they are available, only server information will be passed back as part of additional information in the sub
        structure GatherBasedDatasetInfo inside the key value pair string DataSegment.
        
        
        Use cases:
        Use Case1: Retrieve markup information for MSWordX dataset
        From the Teamcenter client for Microsoft Office Word, when a user opens a WordX Dataset and clicks the Markup
        button from the Teamcenter ribbon, internally, the client initiates the gatherMarkups operation to retrieve
        markup related information.  When the operation completes and returns the data back to the client, the client
        will process this data and display the markup information in the Markup window pane.
        
        Use Case 2: Visualization
        The viewer integrations have the need to gather up all markup objects related to a given object for display in
        the viewer.  An Item, ItemRevision, or dataset is the typical input object, and the return is a list of markup
        datasets that are related.  The server can choose to filter the return based on certain business logic such as
        user access control rights.  For example, it may not be desireable to allow users that do not own the markup
        actually edit it, so these types of controls can be imposed by this service.  A typical use case is as follows:
        1.    Visualization user locates an object that contains a base image (e.g. the ItemRevision representing a
        part with a drawing attached to it) in the PLM client
        2.    The user launches the base image control object to visualization
        3.    The user can control whether or not to Open with markups, depending on how their load preferences are
        setup.  Assume the open with markups behavior is desired for the remainder of this use case.
        4.    The system calls the gatherMarkups service to find all markups related to the base image object of
        interest.
        5.    The gatherMarkups service imposes the appropriate business rules while gathering the related markup
        objects, and returns the appropriate markup datasets that the user should be able to see to the client
        6.    The visualization client opens the base image and the related markup layers, and enables the appropriate
        level of functionality (such as the markup entity controls if the user has edit access).
        """
        return cls.execute_soa_method(
            method_name='gatherMarkups',
            library='Internal-Visualization',
            service_date='2010_04',
            service_name='DataManagement',
            params={'selectedInputInfos': selectedInputInfos, 'serverInfo': serverInfo, 'sessionInfo': sessionInfo, 'userDataAgentInfo': userDataAgentInfo},
            response_cls=GatherResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ModelSchema
from tcsoa.base import TcService


class SessionService(TcService):

    @classmethod
    def initTypeByNames(cls, typeNames: List[str]) -> ModelSchema:
        """
        Get schema info from server
        """
        return cls.execute_soa_method(
            method_name='initTypeByNames',
            library='Internal-Core',
            service_date='2006_03',
            service_name='Session',
            params={'typeNames': typeNames},
            response_cls=ModelSchema,
        )

    @classmethod
    def initTypeByUids(cls, uids: List[str]) -> ModelSchema:
        """
        Get schema info from server
        """
        return cls.execute_soa_method(
            method_name='initTypeByUids',
            library='Internal-Core',
            service_date='2006_03',
            service_name='Session',
            params={'uids': uids},
            response_cls=ModelSchema,
        )



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from tcsoa.gen.BusinessObjects import ImanFile


@dataclass
class FileInfo(TcBaseObj):
    """
    File Info
    
    :var clientFileId: Unique Identifier defined by user to track files
    :var refName: Named Reference relation to file.
    :var isText: Flag to indicate if file is text(TRUE) or binary(FALSE).
    :var fileName: Name of file to be uploaded. Filename with extension only.
    """
    clientFileId: str = ''
    refName: str = ''
    isText: bool = False
    fileName: str = ''


@dataclass
class FileInfoTicket(TcBaseObj):
    """
    Structure containing FMS ticket
    
    :var clientFileId: unique Identifier defined by user to track files
    :var ticket: FMS write ticket for the file uploaded.
    """
    clientFileId: str = ''
    ticket: str = ''


@dataclass
class FileTicketsResponse(TcBaseObj):
    """
    Holds a 'ServiceData' object and a map of write tickets information.  If the request completes successfully, the
    information in the vector can be used to upload the files to  the Teamcenter volume.  Returned from the
    'getWriteTickets' operation.
    
    :var tickets: A map of the 'FileInfoTicket' struct to the clientId.
    :var serviceData: The Teamcenter Services structure used to return status of the operation.  Any errors that
    occurred during the operation are returned here.
    """
    tickets: TicketMap = None
    serviceData: ServiceData = None


@dataclass
class GetFileTransferTicketsResponse(TcBaseObj):
    """
    Holds a 'ServiceData' object, a map (ImanFile, string) of read tickets, and a map (ImanFile, string) of write
    tickets.  If the request completes successfully, each input ImanFile object will appear as a key in both the
    'readTickets' and 'writeTickets' output maps, and the string values associated with each of those entries will be
    valid FMS tickets associated with that ImanFile.
    
    :var readTickets: A map of the input ImanFile objects to FMS read tickets used to read the file from the Default
    Local Volume.  If the ImanFile object is not eligible for StoreAndForward, then this map will not contain a read
    ticket for the ImanFile object.  The map returned may contain no elements.
    :var writeTickets: A map of the input ImanFile objects to FMS write tickets used to write the file to the Default
    Volume.  If the ImanFile object is not eligible for StoreAndForward, then this map will not contain a write ticket
    for the ImanFile object.  The map returned may contain no elements.
    :var serviceData: The Teamcenter Services structure used to return status of the operation.  Any errors that
    occurred during the operation are returned here.
    """
    readTickets: TransferTicketMap = None
    writeTickets: TransferTicketMap = None
    serviceData: ServiceData = None


@dataclass
class GetRegularFileWriteTicketsInput(TcBaseObj):
    """
    Used to input information to the getRegularFileTicketsForUpload()
    function.  The struct holds the array of RegularFileInfo and
    a string client id information.
    
    :var regularFileInfos: The structure holds the file name and text/binary file information for each file to be
    uploaded to a Teamcenter volume.
    :var clientId: The calling client can supply a string to identify each individual file in the maps returned from
    this and related operations.  The 'clientId' should be unique for each ImanFile instance, but this is apparently
    not enforced by this operation.
    """
    regularFileInfos: List[RegularFileInfo] = ()
    clientId: str = ''


@dataclass
class GetRegularFileWriteTicketsResponse(TcBaseObj):
    """
    Used to return information from the getRegularFileTicketsForUpload()
    function.  The struct holds the Servicedata and a map of (string)
    file names and (string) write tickets.
    
    :var writeTickets: Map of (string) file names (from the 'RegularFileInfo' input) to (string) write tickets (output).
    :var serviceData: serviceData
    """
    writeTickets: WriteTicketsMap = None
    serviceData: ServiceData = None


@dataclass
class RegularFileInfo(TcBaseObj):
    """
    Used to input information to the getRegularFileTicketsForUpload()
    function.  The struct holds the file name and text/binary file information.
    
    :var fileName: File name of the file which needs to be uploaded.  This is typically a full path to the file being
    uploaded.
    :var isText: Boolean to define if the file is binary or text file. True if the file is a text file, false otherwise.
    """
    fileName: str = ''
    isText: bool = False


@dataclass
class UpdateImanFileCommitsResponse(TcBaseObj):
    """
    Holds a 'ServiceData' object, a map (ImanFile, integer) of delays, and a map (ImanFile, string) of write tickets
    used to cleanup the files from the initial Store and Forward volume.  If the request completes successfully, an
    ImanFile object corresponding to each input file GUID will appear as a key in both the delays and writeTickets
    output maps, and the string values associated with each of the writeTickets entries will be valid FMS read tickets
    associated with that ImanFile.  See the structure description for more information.
    
    :var delays: A mapping of ImanFile objects to the time delay before the tickets corresponding to that ImanFile
    object times out.  If the corresponding data file has not been successfully transferred to the new volume and
    committed to the new volume, then this map will not contain a delay value for the ImanFile object.  The map
    returned may contain no elements.
    :var writeTickets: A mapping of ImanFile objects to FMS write tickets used to remove the transferred file from the
    Default Local Volume in the cleanup dispatcher task.  If the corresponding data file has not been successfully
    transferred to the new volume and committed to the new volume, then this map will not contain a write ticket for
    the ImanFile object.  The map returned may contain no elements.
    :var serviceData: The Teamcenter Services structure used to return status of the operation.  Any errors that
    occurred during the operation are returned here.
    """
    delays: DelayMap = None
    writeTickets: TransferTicketMap = None
    serviceData: ServiceData = None


@dataclass
class WriteTicketsInput(TcBaseObj):
    """
    Holds information about the Dataset (datasettype, version) and the list of files (marked by unique identifier) to
    be uploaded in the desired datasetType.
    
    :var clientId: Unique identifier defined by user to track each file.
    :var datasetTypeName: Desired datasetType of the Dataset to upload the files.
    :var version: Set this value to one(1) in the use case where a new Dataset is going to be created.  Otherwise, it
    should be set to one more than the current highest version number of the existing Dataset, which must be determined
    by the caller. This value is used for filename generation in the Teamcenter volume. (Note that the version number
    and the version limit are not necessarily the same.)
    :var fileInfos: A vector of 'FileInfo' structures where each contains information specific to each file that will 
    be uploaded to the Dataset via Visualization services.
    """
    clientId: str = ''
    datasetTypeName: str = ''
    version: int = 0
    fileInfos: List[FileInfo] = ()


@dataclass
class CommitUploadedRegularFilesInput(TcBaseObj):
    """
    Used to input information to the commitRegularFiles()
    function.  The struct holds string file name, string file ticket and
    a string client id information.
    
    :var fileName: fileName
    :var fileTicket: fileTicket
    :var clientId: clientId
    """
    fileName: str = ''
    fileTicket: str = ''
    clientId: str = ''


@dataclass
class CommitUploadedRegularFilesResponse(TcBaseObj):
    """
    Used to return information from the commitRegularFiles()
    function.  The struct holds the Servicedata and a map of (string)
    file names and ImanFileImpl objects.
    
    :var files: files
    :var serviceData: serviceData
    """
    files: ImanFilesMap = None
    serviceData: ServiceData = None


"""
A mapping of ImanFile objects to the time delay before the tickets corresponding to that ImanFile object times out.  If the corresponding data file has not been successfully transferred to the new volume and committed to the new volume, then this map will not contain a delay value for the ImanFile object.  The map returned may contain no elements.
"""
DelayMap = Dict[ImanFile, int]


"""
Map of (string) file names and ImanFile objects.
"""
ImanFilesMap = Dict[str, ImanFile]


"""
A map of the 'FileInfoTicket' struct to the clientId.
"""
TicketMap = Dict[str, List[FileInfoTicket]]


"""
A map of the input ImanFile objects to FMS read tickets used to read the file from the Default Local Volume or write tickets used to write the file to the Default Volume, or write tickets used to remove the transferred file from the Default Local Volume in the cleanup dispatcher task (depending on the context in which this map is used).  If the ImanFile object is not eligible for StoreAndForward, or has not been successfully transferred to the new volume and committed to the new volume, then this map will not contain a read or write ticket for the ImanFile object.  The map returned may contain no elements.
"""
TransferTicketMap = Dict[ImanFile, str]


"""
Map of (string) file names (from the 'RegularFileInfo' input) to (string) write tickets (output).
"""
WriteTicketsMap = Dict[str, str]



from tcsoa.gen.Internal_Vendormanagement._2008_05.services import VendorManagementService as imp0
from tcsoa.gen.Internal_Vendormanagement._2008_06.services import VendorManagementService as imp1
from tcsoa.base import TcService


class VendorManagementService(TcService):
    createOrUpdateLineItems = imp0.createOrUpdateLineItems
    getItemIDwithContext = imp0.getItemIDwithContext
    getVPSRConditions = imp1.getVPSRConditions



from __future__ import annotations

from tcsoa.gen.Internal.AWS2._2016_03.Finder import ColumnConfigInput
from tcsoa.gen.Internal.AWS2._2019_12.Finder import FilterFacetInput, StringMap4, FilterFacetResponse, SearchInput4, ExportObjectsToFileResponse, PropertyInfo
from tcsoa.gen.Internal.AWS2._2019_12.DataManagement import IDContextOutput, IdentifierTypesOut, IdentifierTypesIn, IDDispRuleCreateIn
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService
from tcsoa.gen.BusinessObjects import WorkspaceObject


class DataManagementService(TcService):

    @classmethod
    def createIdDisplayRules(cls, idDispRuleCreIns: List[IDDispRuleCreateIn]) -> ServiceData:
        """
        This operation creates the ID Display Rules (IdDispRule) with the input names and the corresponding ID Context
        information. ID Display Rule can also be created without the any input ID Context and user can update the
        object later with ID Context information.
        ID Display Rule contains the list of ID Contexts and their order. Based on the order of the ID Contexts
        defined, the system evaluates the display name of the Item and ItemRevision from their Alternate IDs.
        
        ID Context (IdContext), represents the context information under which the OEM defines the unique IDs of their
        Item and ItemRevision. This context information is used when Teamcenter users define the unique IDs of Item and
        ItemRevision objects.
        
        User can set one of the ID Display Rules as the current ID Display Rule. The current ID Display Rule is used to
        evaluate the display names of the Item and ItemRevision. In case the ID Context of the Alternate ID with the
        Item and ItemRevision object does not match with that of the current ID Display Rule then system uses the
        default ID Display Rule to evaluate the display names of Item and ItemRevision objects.
        """
        return cls.execute_soa_method(
            method_name='createIdDisplayRules',
            library='Internal-AWS2',
            service_date='2019_12',
            service_name='DataManagement',
            params={'idDispRuleCreIns': idDispRuleCreIns},
            response_cls=ServiceData,
        )

    @classmethod
    def getIdContexts(cls, inputObjs: List[WorkspaceObject]) -> IDContextOutput:
        """
        This operation fetches all instances of the ID Context objects (IdContext) from the Teamcenter database
        applicable for the input objects of type Item and ItemRevision based on defined ID Context Rules
        (IdContextRule) by the system administrators.
        
        This operation queries ID Context Rule objects and fetches the ID Context objects based on the input Item,
        ItemRevision or NULL. The input is the identifiable type defined on the ID Context Rules. For a NULL input, it
        returns the Id Context objects where the identifiable type is NULL.
        All ID Context objects from the Teamcenter data base are returned in case input object is other than Item,
        ItemRevision or NULL. An empty input list would also return all ID Context objects from the Teamcenter data
        base.
        
        IdContext objects represents the context information under which the OEM defines the unique IDs of their Item
        and ItemRevision objects. This context information is used when Teamcenter users define the unique IDs of Item
        and ItemRevision objects.
        
        Alternate and Alias IDs of Teamcenter are the example of the such unique IDs of Item and ItemRevision. Users
        define Alternate and Alias IDs with the help of the ID Context as one of the unique attribute of the ID.
        """
        return cls.execute_soa_method(
            method_name='getIdContexts',
            library='Internal-AWS2',
            service_date='2019_12',
            service_name='DataManagement',
            params={'inputObjs': inputObjs},
            response_cls=IDContextOutput,
        )

    @classmethod
    def getIdentifierTypes(cls, identifierTypesIn: List[IdentifierTypesIn]) -> IdentifierTypesOut:
        """
        This operation fetches the applicable Identifier types for the input objects of type Item and/or ItemRevision
        along with the input IdContext object. System queries the ID Context Rules defined in Teamcenter database and
        retrives the Identifier types.
        
        Alternate and Alias IDs are defined in Teamcenter as instances of business object of type Identifier. ID
        Context, of business object type IdContext, represents the context information under which the OEM defines the
        unique IDs of their Item and ItemRevision. This context information is used when Teamcenter users define the
        Alternate and Alias IDs of Item and ItemRevision objects.
        
        ID Context Rules are defined as instances of business object type IdContextRule in Teamcenter database. These
        rules map the combination of ID Context and the object type e.g.  Item or ItemRevision, called Identifiable
        types, to the type of the Identifier applicable in the context.
        
        This operation also returns the other applicable objects for which Alternate IDs along with the input objects
        can be defined. In case of input objects of type Item, this operation returns the list of revision objects of
        the Item, and in case of input objects of type ItemRevision, this operation returns the Item object as the
        applicable object for which Alternate IDs can be defined.
        """
        return cls.execute_soa_method(
            method_name='getIdentifierTypes',
            library='Internal-AWS2',
            service_date='2019_12',
            service_name='DataManagement',
            params={'identifierTypesIn': identifierTypesIn},
            response_cls=IdentifierTypesOut,
        )


class FinderService(TcService):

    @classmethod
    def exportObjectsToFile(cls, searchInput: SearchInput4, objectsToExport: List[str], attributesToExport: List[PropertyInfo], columnConfigInput: ColumnConfigInput, exportOptions: StringMap4) -> ExportObjectsToFileResponse:
        """
        This operation exports either a given list of objects or the objects found through the given search input. If
        searchInput and objectsToExport are both present in the input, then objectsToExport will take precedence. The
        objects are exported in a tabular format, with the objects as the rows and the given attributesToExport as the
        columns. If an object does not contain an attribute in attributesToExport then the value that will be exported
        is an empty string. There is also no relation between attributesToExport and columnsToExclude from
        columnConfigInput. 
        
        This operation allows callers to specify export options, which determines the behavior of the export operation.
        The only supported export option for AW4.3 is "fileFormat":"MSExcel" but will be extended in future releases.
        
        Use cases:
        1) User clicks the export command in an object set, which launches the export properties panel. User selects
        the desired properties to export and clicks the export button. Then the objects and the selected properties are
        exported into a table into Excel.
        """
        return cls.execute_soa_method(
            method_name='exportObjectsToFile',
            library='Internal-AWS2',
            service_date='2019_12',
            service_name='Finder',
            params={'searchInput': searchInput, 'objectsToExport': objectsToExport, 'attributesToExport': attributesToExport, 'columnConfigInput': columnConfigInput, 'exportOptions': exportOptions},
            response_cls=ExportObjectsToFileResponse,
        )

    @classmethod
    def getFilterValues(cls, filterFacetInput: FilterFacetInput) -> FilterFacetResponse:
        """
        This operation fetches the possible filter values (Facets) of a given column in a table. Each table in Active
        Workspace is typically backed by a provider, so the input of this operation is similar to performSearch
        operation&rsquo;s searchInput request. 
        The framework allows a custom solution to be able to provide a new specific provider to collect data, perform
        sorting and filtering. The provider can be a User Inbox retriever or Full Text searcher, for example. The new
        data provider can be encapsulated via a new runtime business object from Fnd0BaseProvider class. The
        implementation is done using its fnd0performSearch operation.
        
        RuntimeBusinessObject 
        ---- Fnd0BaseProvider (foundation template) 
        -------- Fnd0GetChildrenProvider(foundation template) 
        -------- Awp0FullTextSearchProvider (aws template)
        -------- Awp0TaskSearchProvider (aws template) 
        -------- Aos0TestProvider (aosinternal template) 
        -------- etc.
        
        This operation provides a common interface to get Filter Facet values in a generic way from any data provider
        that is compatible with performSearch operation. This operation takes the data provider which is responsible
        for fetching the objects, a columnName one which the filter values need to be obtained, and a filter or query
        criteria which is used to refine the filter facet values. Once the query or filtering is complete, the ui
        values are obtained from the search results and are returned as the facet values.
        
        Use cases:
        User clicks on "Name" column in Home Folder and clicks on Show Filters to expand the possible Filter facet
        values
        A getFilterValues request will be made by passing in the needed query or filter criteria. In this case the
        provider name is "Awp0ObjectSetRowProvider". The provider is responsible for fetching the objects of the Home
        Folder, then it queries for the Name property of all the objects returned by the data provider and returns a
        unique sorted UI values for the "Name" column. These are the facet results and are shown in a list of
        checkboxes in a column menu in the client.
        """
        return cls.execute_soa_method(
            method_name='getFilterValues',
            library='Internal-AWS2',
            service_date='2019_12',
            service_name='Finder',
            params={'filterFacetInput': filterFacetInput},
            response_cls=FilterFacetResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class SearchFailure(TcBaseObj):
    """
    This structure returns failure during publish/unpublish operation
    
    :var failureObject: The object that failed to publish.
    :var failedSite: The name of the site the object failed to publish to.
    :var failureCodes: A list of failure codes for the failureObject.
    :var failureStrings: A list of error strings corresponding to each error in failureCodes.
    """
    failureObject: BusinessObject = None
    failedSite: str = ''
    failureCodes: List[int] = ()
    failureStrings: List[str] = ()


@dataclass
class SearchResponse(TcBaseObj):
    """
    Return structure for publish/unpublish operation
    
    :var failureInfo: A list of SearchFailure structures. Each structure contains failure information for a specific
    business object that was supplied.
    :var serviceData: The standard ServiceData return.
    """
    failureInfo: List[SearchFailure] = ()
    serviceData: ServiceData = None


"""
Map of string array property names to values (string, vector).
"""
StringVectorMap1 = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ScheduleTask, SchDeliverable, Schedule
from typing import List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class MembershipData(TcBaseObj):
    """
    The information needed to create a new member in a schedule.
    
    :var schedule: The schedule for the new membership.
    :var resource: The resource to add.  (This can be a UserLogin, Group, or Discipline).
    :var membershipLevel: The membership level in that schedule.
    """
    schedule: Schedule = None
    resource: BusinessObject = None
    membershipLevel: int = 0


@dataclass
class ScheduleCopyContainer(TcBaseObj):
    """
    The information need to copy a schedule.
    
    :var name: The name of the new schedule.
    :var scheduleToCopy: The tag of the schedule to copy.
    :var resetWork: Flag to indicate whether or not to reset the tasks' execution data (%, status, work complete, etc).
    :var copyBaselines: Flag to indicate whether or not to copy baselines.
    """
    name: str = ''
    scheduleToCopy: Schedule = None
    resetWork: bool = False
    copyBaselines: bool = False


@dataclass
class ScheduleDeliverableData(TcBaseObj):
    """
    A container for a new Schedule Deliverable.
    
    :var schedule: The schedule to contain this deliverable.
    :var deliverableType: The type of the deliverable.
    :var deliverableName: The name of the deliverable.
    """
    schedule: Schedule = None
    deliverableType: str = ''
    deliverableName: str = ''


@dataclass
class ScheduleObjDeleteContainer(TcBaseObj):
    """
    An object to delete.
    
    :var object: A tag to the object to be deleted.
    """
    object: BusinessObject = None


@dataclass
class StringValContainer(TcBaseObj):
    """
    The container represents a single attribute's value. 
    The name of the attribute is represented by the key field. 
    The data type of the attribute is represented by the type field. The type field can be one of the following;
    char = 0;
    date = 1;
    double = 2;
    float = 3;
    int = 4;
    bool = 5;
    short = 6;
    string = 7;
    ModelObject = 8;
    
    :var key: A string key identifying the attribute.
    :var value: A string representation of the value of the attribute.
    :var type: An integer to help determine the type of object represented by the value.
    """
    key: str = ''
    value: str = ''
    type: int = 0


@dataclass
class TaskDeliverableData(TcBaseObj):
    """
    The input information for a single task deliverable.
    
    :var scheduleTask: The task which will contain this delievable.
    :var scheduleDeliverable: The ScheduleDeliverable to reference.
    :var submitType: The submit type  (3=Don't submit, 0=submit as target, 1=submit as reference).
    """
    scheduleTask: ScheduleTask = None
    scheduleDeliverable: SchDeliverable = None
    submitType: int = 0


@dataclass
class CreateBaselineContainer(TcBaseObj):
    """
    The information to create a baseline.
    
    :var name: The name of the baseline. If empty, uses the schedule ID of the baseline.
    :var schedule: The 'Schedule' to baseline.
    :var parentBaseline: The 'Schedule' baseline to copy from.
    :var isActive: If true, then set the newly created 'baseline' as the active 'baseline'.
    :var includeNewTasks: If true, then baseline the newly created tasks.
    :var updateScheduleBaselineCost: If true, update the baselined 'Schedule' cost. (Not currently used)
    :var taskRebaseOption: A bitmask of types of task to rebaseline (0=none, 1=non-started, 2=non-complete).
    """
    name: str = ''
    schedule: Schedule = None
    parentBaseline: Schedule = None
    isActive: bool = False
    includeNewTasks: bool = False
    updateScheduleBaselineCost: bool = False
    taskRebaseOption: int = 0


@dataclass
class CreateScheduleContainer(TcBaseObj):
    """
    The container for a new schedule.
    
    :var name: The name. Valid value- can not be null.
    :var description: The description. Valid value- can be empty string.
    :var datesLinked: areDatesLinked?
    :var percentLinked: isPercentLinked?
    :var isTemplate: isTemplate?
    :var scheduleType: The schedule type. Valid value (0-Standard)
    :var isPublic: isPublic?
    :var stringValueContainer: A collection of additional attributes.(optional)
    :var customerName: The customer's name. Valid value- can be empty string.
    :var customerNumber: The customer's ID. Valid value- can be empty string.
    :var startDate: The start date.
    :var finishDate: The finish date.
    :var priority: The priority. Valid values are {0-lowest,1-low,2-MediumLow,3-Medium,4-High,5-VeryHigh,6-Highest}
    :var status: The status. Valid values are { 0-Not
    started,1-InProgress,2-NeedsAttention,3-Complete,4-Abandoned,5-Late}
    :var published: isPubished?
    :var linksAllowed: areLinksAllowed?
    """
    name: str = ''
    description: str = ''
    datesLinked: bool = False
    percentLinked: bool = False
    isTemplate: bool = False
    scheduleType: int = 0
    isPublic: bool = False
    stringValueContainer: List[StringValContainer] = ()
    customerName: str = ''
    customerNumber: str = ''
    startDate: datetime = None
    finishDate: datetime = None
    priority: int = 0
    status: str = ''
    published: bool = False
    linksAllowed: bool = False



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class AccCheckInput(TcBaseObj):
    """
    This structure provides a set of input values for accountabilityCheck operation.
    
    :var sourceTags: the source BOM lines
    :var targetTags: the target BOM lines
    :var option: options of search
    :var srcCtxtLineTag: the possible source context line
    :var tarCtxtLineTag: the possible target context line
    :var matchType: represents user choice in the color display
    """
    sourceTags: List[BusinessObject] = ()
    targetTags: List[BusinessObject] = ()
    option: int = 0
    srcCtxtLineTag: BusinessObject = None
    tarCtxtLineTag: BusinessObject = None
    matchType: int = 0


@dataclass
class AccountabilityCheckResponse(TcBaseObj):
    """
    This structure provides response of the call
    
    :var resultSourceTarget: Accountabilty Check result
    :var serviceData: Partial errors
    """
    resultSourceTarget: List[AccountabilityCheckResult] = ()
    serviceData: ServiceData = None


@dataclass
class AccountabilityCheckResult(TcBaseObj):
    """
    The structure characterizes compare result for each source line
    
    :var source: the sourse BOM line
    :var targets: the target BOM lines
    :var checkResult: characterized check result
    """
    source: BusinessObject = None
    targets: List[BusinessObject] = ()
    checkResult: int = 0



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from tcsoa.gen.BusinessObjects import BusinessObject


@dataclass
class GetProductScopeForProcessResponse(TcBaseObj):
    """
    Response of getProductScopeForProcess operation.
    Returns a list of product line from BOM structure which are connected to the given process line with relation
    "Fnd0ProcessToScopeRel".
    
    The following partial errors may be returned as part of service data
    &bull;    200603     Invalid object type for process line.
    
    :var scopedProductBOMLine: list of map < Teamcenter::BusinessObject, std::vector<Teamcenter:: BusinessObject> >
    containing the input process line as key and a list of product line as value.
    :var serviceData: service data containing partial errors.
    """
    scopedProductBOMLine: List[InputProcLineToProductLinesMap] = ()
    serviceData: ServiceData = None


"""
A Map <BusinessObject, std::vector<BusinessObject> > contains the input process line as key and a list of related product lines from BOM structure as value.
"""
InputProcLineToProductLinesMap = Dict[BusinessObject, List[BusinessObject]]



from __future__ import annotations

from enum import Enum
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from tcsoa.gen.BusinessObjects import WorkspaceObject
from dataclasses import dataclass


@dataclass
class ReleaseStatusInput(TcBaseObj):
    """
    ReleaseStatus input
    
    :var operations: Operations to perform ( Currently only Append is supported )
    :var objects: Objects to modify
    """
    operations: List[ReleaseStatusOption] = ()
    objects: List[WorkspaceObject] = ()


@dataclass
class ReleaseStatusOption(TcBaseObj):
    """
    ReleaseStatus option
    
    :var newReleaseStatusTypeName: Name of release type to instantiate and assign
    :var operation: Operation to perform
    :var existingreleaseStatusTypeName: Name of old release type to delete or replace
    """
    newReleaseStatusTypeName: str = ''
    operation: releaseStatusOperation = None
    existingreleaseStatusTypeName: str = ''


@dataclass
class SetReleaseStatusResponse(TcBaseObj):
    """
    service data
    
    :var serviceData: serviceData
    """
    serviceData: ServiceData = None


class releaseStatusOperation(Enum):
    """
    releaseStatusOperation
    """
    Append = 'Append'
    Delete = 'Delete'
    Rename = 'Rename'
    Replace = 'Replace'



from __future__ import annotations

from tcsoa.gen.Internal.ImportExport._2017_11.FileImportExport import ImportDocumentInputData
from tcsoa.gen.ImportExport._2011_06.FileImportExport import ImportFromApplicationResponse1
from typing import List
from tcsoa.base import TcService


class FileImportExportService(TcService):

    @classmethod
    def importDocumentAsync(cls, input: ImportDocumentInputData) -> None:
        """
        This operation imports the selected Requirement Specification word document asynchronously to Teamcenter.
        Import will be done using keywords specified by user and will be done in the background.
        """
        return cls.execute_soa_method(
            method_name='importDocumentAsync',
            library='Internal-ImportExport',
            service_date='2017_11',
            service_name='FileImportExport',
            params={'input': input},
            response_cls=None,
        )

    @classmethod
    def importDocumentOffline(cls, inputs: List[ImportDocumentInputData]) -> ImportFromApplicationResponse1:
        """
        This operation imports a Requirement Specification document containing Requirement objects and related
        information. It creates SpecElement objects and saves associated data (FullText for each created object to
        store the content from the document and IMAN_Specification relation between FullText and each created object).
        The input structure for this operation contains type of specification elements (SpecElement) to be created,
        keyword parsing rules to be used during import, option to import as new specification or under the selected
        object and description to be set on the Specification once imported.
        """
        return cls.execute_soa_method(
            method_name='importDocumentOffline',
            library='Internal-ImportExport',
            service_date='2017_11',
            service_name='FileImportExport',
            params={'inputs': inputs},
            response_cls=ImportFromApplicationResponse1,
        )



from tcsoa.gen.Rdv._2008_05.services import ContextManagementService as imp0
from tcsoa.gen.Rdv._2012_09.services import ContextManagementService as imp1
from tcsoa.gen.Rdv._2010_09.services import ContextManagementService as imp2
from tcsoa.gen.Rdv._2009_04.services import ContextManagementService as imp3
from tcsoa.gen.Rdv._2013_05.services import ContextManagementService as imp4
from tcsoa.base import TcService


class ContextManagementService(TcService):
    addPartToProduct = imp0.addPartToProduct
    createFormAttrSearchCriteria = imp1.createFormAttrSearchCriteria
    createSCO = imp2.createSCO
    createSearchCriteriaScope = imp1.createSearchCriteriaScope
    createSearchSCO = imp1.createSearchSCO
    getAllGOPartSolutions = imp3.getAllGOPartSolutions
    getICSClassNames = imp1.getICSClassNames
    getPastePrimeAttributes = imp3.getPastePrimeAttributes
    getProductItemInfo = imp0.getProductItemInfo
    getRelatedObjectsInContext = imp4.getRelatedObjectsInContext
    removePartsRelatedToABE = imp0.removePartsRelatedToABE
    replacePartInProduct = imp0.replacePartInProduct
    updateSCO = imp2.updateSCO
    updateSearchSCO = imp1.updateSearchSCO



from __future__ import annotations

from tcsoa.gen.ProjectManagement._2008_06.ScheduleManagement import ScheduleCopyOptionsContainer, CreateBillRateResponse, NewScheduleContainer, ScheduleDeliverableData, AddMembershipResponse, TaskCostUpdate, BillRateContainer, UpdateTaskCostDataResponse, MembershipData, ScheduleCopyResponses, CreateScheduleResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ScheduleManagementService(TcService):

    @classmethod
    def copySchedules(cls, scheduleCopyContainer: List[ScheduleCopyOptionsContainer]) -> ScheduleCopyResponses:
        """
        This operation makes a deep copy of the schedule with options to reset work and copy existing baselines. The
        information needed to copy schedule is specified in the 'ScheduleCopyOptionsContainer' structure. It returns
        'ScheduleCopyResponses' which will have information of copied  schedules and 'ServiceData'. Errors will be
        returned in the list of partial errors in the 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='copySchedules',
            library='ProjectManagement',
            service_date='2008_06',
            service_name='ScheduleManagement',
            params={'scheduleCopyContainer': scheduleCopyContainer},
            response_cls=ScheduleCopyResponses,
        )

    @classmethod
    def updateTaskCostData(cls, updates: List[TaskCostUpdate]) -> UpdateTaskCostDataResponse:
        """
        This operation creates, updates and deletes fixed cost entries and task costing metadata such as bill code, sub
        code etc. This operation throws exceptions that are system and database exceptions. There are no specific
        business logic errors. The created objects are returned back in the service data of the response.
        
        """
        return cls.execute_soa_method(
            method_name='updateTaskCostData',
            library='ProjectManagement',
            service_date='2008_06',
            service_name='ScheduleManagement',
            params={'updates': updates},
            response_cls=UpdateTaskCostDataResponse,
        )

    @classmethod
    def createBillRates(cls, rates: List[BillRateContainer]) -> CreateBillRateResponse:
        """
        This operation creates a new BillRates. BillRate is a business object  that is used to represent Rate
        Modifiers,which are used  with resource costing information to calculate schedule and task costs and are
        defined by billing types, rates and currency. This operation throws exceptions that are system and database
        exceptions . There are no specific business logic errors. The created objects are returned back in the service
        data of the response.
        """
        return cls.execute_soa_method(
            method_name='createBillRates',
            library='ProjectManagement',
            service_date='2008_06',
            service_name='ScheduleManagement',
            params={'rates': rates},
            response_cls=CreateBillRateResponse,
        )

    @classmethod
    def addMemberships(cls, membershipData: List[MembershipData]) -> AddMembershipResponse:
        """
        This operation adds resources to the schedule with given membership levels. 
        
        The information required to add  new resource  to the schedule is passed to the function through
        'MembershipData' structure.
        
        The operation saves the references to the newly created membership objects and errors if any in the
        'ServiceData' of the 'MembershipDataResponse' data structure.
         
        When a resource that needs to be added doesnot exist,the operation returns and  the error is saved in the
        'ServiceData' of the response.
        """
        return cls.execute_soa_method(
            method_name='addMemberships',
            library='ProjectManagement',
            service_date='2008_06',
            service_name='ScheduleManagement',
            params={'membershipData': membershipData},
            response_cls=AddMembershipResponse,
        )

    @classmethod
    def createSchedule(cls, newSchedules: List[NewScheduleContainer]) -> CreateScheduleResponse:
        """
        This operation creates new schedule based on the initial user's request to the application interface. The
        information needed to create Schedule is specified in the 'NewScheduleContainer' structure. It returns
        'CreateScheduleResponse' which will have information of created schedules and 'ServiceData'. Errors will be
        returned in the list of partial errors in the 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='createSchedule',
            library='ProjectManagement',
            service_date='2008_06',
            service_name='ScheduleManagement',
            params={'newSchedules': newSchedules},
            response_cls=CreateScheduleResponse,
        )

    @classmethod
    def createScheduleDeliverableTemplates(cls, scheduleDeliverableData: List[ScheduleDeliverableData]) -> ServiceData:
        """
        This operation creates new schedule deliverable. The created objects are returned back in the 'ServiceData' of
        the response. 'ServiceData' will contain partial errors in case of operation failure.
        """
        return cls.execute_soa_method(
            method_name='createScheduleDeliverableTemplates',
            library='ProjectManagement',
            service_date='2008_06',
            service_name='ScheduleManagement',
            params={'scheduleDeliverableData': scheduleDeliverableData},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ImanFile, PSBOMViewRevision, Fnd0BOMLineLite, Dataset, BOMWindow
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ArrAbsOccDataOverride(TcBaseObj):
    """
    The data structure contains occurrence override data for an arrangement&rsquo;s occurrence. If there are no
    overrides for the occurrence (possible in the case of a base arrangement), the override type is 0. 
    There can be only one transform override on an occurrence per arrangement, and only one suppression override. If
    the occurrence is a subassembly, there may also be at most one used arrangement identified.
    For a base arrangement, there may be multiple transform overrides and there may be multiple geometry (jt) overrides.
    
    :var occurrenceThreadPath: A list of occurrence thread UID strings representing the occurrence path. Beginning with
    the specified occurrence up to the occurrence immediately under the context PSBOMViewRevision (i.e. the input
    PSBOMViewRevision.)
    :var overrideTypes: The type of data held in the structure. Values as a bitmask: 0=No override, 1=Transform,
    2=Suppression, 4=Arrangement, 8=JT. For example, if there is both a transform and suppression override, the type
    value is 3.
    :var absTransform: Transformation (positioning) override data of the PSOccurrence.
    Transformation data is provided in a matrix in eXT format of
    { R00 R10 R20 P0 R01 R11 R21 P1 R02 R12 R22 P2 T0 T1 T2 S }
    where the components of this matrix are:
    A general rotation submatrix, comprising
       x-axis (R00, R01, R02),
       y-axis (R10, R11, R12),
       z-axis (R20, R21, R22).
    A perspective (P0, P1, P2).
    A translation (T0, T1, T2).
    A scaling factor (S).
    :var suppression: The occurrence suppression setting, in the context of the arrangement.
    If true, the occurrence is not displayed.
    :var usedArrangement: Identifier for an arrangement used by a child subassembly.
    :var jtOverride: The geometry (jt) data override in context of the input PSBOMViewRevision.
    """
    occurrenceThreadPath: List[str] = ()
    overrideTypes: int = 0
    absTransform: List[float] = ()
    suppression: bool = False
    usedArrangement: ArrangementID = None
    jtOverride: ArrJtOverride = None


@dataclass
class ArrJtOverride(TcBaseObj):
    """
    The data structure contains information about the absolute occurrence override of geometry (jt) data.
    
    :var jtDataset: The dataset object reference of the geometry (jt) information.
    :var jtFile: The file object reference of the geometry (jt) information
    :var jtFileName: Original filename.
    :var fileTicket: FMS ticket to use in file upload.
    """
    jtDataset: Dataset = None
    jtFile: ImanFile = None
    jtFileName: str = ''
    fileTicket: str = ''


@dataclass
class ArrOccurrenceData(TcBaseObj):
    """
    The data structure contains occurrence and override data for an occurrence of the arrangement. The relative
    transformation information is only provided for base arrangements. In a base arrangement, the override data is not
    in context of any "named" arrangement.
    
    :var occurrenceThread: The occurrence thread UID string.
    :var occTransform: Transformation (positioning) override data of the PSOccurrence. 
    Transformation data is provided in a matrix in eXT format of
    { R00 R10 R20 P0 R01 R11 R21 P1 R02 R12 R22 P2 T0 T1 T2 S }
    where the components of this matrix are:
    A general rotation submatrix, comprising
       x-axis (R00, R01, R02),
       y-axis (R10, R11, R12),
       z-axis (R20, R21, R22).
    A perspective (P0, P1, P2).
    A translation (T0, T1, T2).
    A scaling factor (S).
    :var absOccOverrides: List of override data from the AbsOccData for an absolute occurrence.
    """
    occurrenceThread: str = ''
    occTransform: List[float] = ()
    absOccOverrides: List[ArrAbsOccDataOverride] = ()


@dataclass
class ArrangementData(TcBaseObj):
    """
    The data structure contains data of the "base" arrangement or of a single AssemblyArrangement, for a specified
    product structure.  For a base arrangement, the arrangement ID is empty, and the occurrence data list contains
    elements for each immediate child of the product structure.
    
    :var arrangeType: The type of arrangement. Supported values are: 0=Base, 1=Default, 2=As_Saved, and 3=Named.
    :var arrangeID: Data to uniquely identify an AssemblyArrangement of the structure.
    :var occData: List of occurrence and override information for each occurrence in an arrangement.
    """
    arrangeType: int = 0
    arrangeID: ArrangementID = None
    occData: List[ArrOccurrenceData] = ()


@dataclass
class ArrangementID(TcBaseObj):
    """
    The data structure contains information to uniquely identify an arrangement of a structure.
    
    :var arrangeName: Name of AssemblyArrangement. (May be empty.)
    :var arrangeSubfileid: External UID of AssemblyArrangement.
    """
    arrangeName: str = ''
    arrangeSubfileid: str = ''


@dataclass
class GetProductStructureArrangementsResp(TcBaseObj):
    """
    The data structure contains output corresponding to the input list of product structures 
    (identified by PSBOMViewRevisions or Fnd0BOMLineLite lines), along with the ServiceData of the operation.
    
    :var arrOutput: List of ProductStructureArrOutput objects.
    :var serviceData: Contains PartialErrors.
    """
    arrOutput: List[ProductStructureArrOutput] = ()
    serviceData: ServiceData = None


@dataclass
class LWBStructureInput(TcBaseObj):
    """
    Identifies the structures for which to return arrangement data, in the LWB (Light Weight BOM) scenario.
    
    :var lwbWindow: Window in which the product structure is expanded
    :var lwbLines: List of lines contained in lwbWindow from which to obtain arrangement data.
    """
    lwbWindow: BOMWindow = None
    lwbLines: List[Fnd0BOMLineLite] = ()


@dataclass
class ProductStructureArrOutput(TcBaseObj):
    """
    The data structure contains arrangement data for the specified product structures.
    
    :var psArrOutput: List of arrangement data from all the structures&rsquo; arrangements.
    """
    psArrOutput: List[ProductStructureArrangementsOutput] = ()


@dataclass
class ProductStructureArrangementInput(TcBaseObj):
    """
    Identifies the product structures for which to return arrangement data. The expected input is either a list of
    specific PSBOMViewRevisions; or a BOMWindow and list of corresponding Fnd0BOMLineLite lines.
    
    :var bvrs: List of PSBOMViewRevisions from which to obtain arrangement data.
    :var lwbStructures: Identifies the product structures from which to return arrangement data.
    """
    bvrs: List[PSBOMViewRevision] = ()
    lwbStructures: LWBStructureInput = None


@dataclass
class ProductStructureArrangementsOutput(TcBaseObj):
    """
    The data structure contains arrangement data for the specified product structure, identified by either bvr or line.
    The base arrangement is the first element in the arrangement list.
    
    :var bvr: The PSBOMViewRevision to which the arrangement data belongs, it is the "parent context".
    :var line: The Fnd0BOMLineLite line to which the arrangement data belongs, it is the "parent context".
    :var arrangeData: List of arrangement data from each of the structure&rsquo;s arrangements.
    """
    bvr: PSBOMViewRevision = None
    line: Fnd0BOMLineLite = None
    arrangeData: List[ArrangementData] = ()
